
!HTTPPost methodsFor: 'private-parsing support' stamp: 'PaoloBonzini 10/4/2009 11:50'!
blockStreamingFrom: aSwazooStream to: outStream until: aBoundaryBytes "detected"	"copy by 8k blocks for optimal performance until a boundary of MIME part is detected"
	"Finish appropriatelly streaming at the end (skip crlf etc.)"
	| boundary start nrOfBoundary inPrevious remainingBoundary boundaryIndex |
	boundary := aBoundaryBytes asByteArray.
	start := self readPosition.
	[true] whileTrue:
		[nrOfBoundary := 0.
		[nrOfBoundary = 0] whileTrue:
			[nrOfBoundary := aSwazooStream signsOfBoundary: boundary.
			nrOfBoundary = 0 ifTrue: "no boundary in current buffer content"
				[self incReadPosition:
					(aSwazooStream nextPutAllBufferOn: outStream).
				self checkToEnlargeBufferIn: aSwazooStream from: start. "for effective streaming"
				aSwazooStream fillBuffer] ].
		"copy and stream out content up to potential boundary"
		boundaryIndex := aSwazooStream indexOfBoundary: boundary.
		inPrevious := aSwazooStream copyBufferTo: boundaryIndex.
		outStream nextPutAll: 
			(inPrevious copyFrom: 1 to: (inPrevious size-2 max: 0) "without potential crlf"). 
		self incReadPosition: inPrevious size. "potential crlf included!!"

		nrOfBoundary = boundary size ifTrue: "full boundary detected, lets finish here"
			[aSwazooStream skip: boundary size. "skip boundary"
			self incReadPosition: boundary size.
			^true].  "streaming complete"

		self incReadPosition: nrOfBoundary.
		aSwazooStream fillBuffer. "let's get next buffer"
		remainingBoundary := boundary copyFrom: nrOfBoundary+1 to: boundary size.
		(aSwazooStream startsWith:  remainingBoundary) ifTrue: "bound. ends in next buff?"
			[aSwazooStream skip: remainingBoundary size + 2.  "skip remaining bound. and crlf"
			self incReadPosition: remainingBoundary size + 2.
			^true]. "streaming complete"

		outStream 
			nextPutAll: (inPrevious copyFrom: inPrevious size-2 to: inPrevious size);  "potential crlf"
			nextPutAll: (boundary copyFrom: 1 to: nrOfBoundary).  "boundary part in prev.buff."
		] "continue from the start"! !

!HTTPPost methodsFor: 'private-parsing support' stamp: 'PaoloBonzini 10/4/2009 10:39'!
checkToEnlargeBufferIn: aSwazooStream from: startPosition
	aSwazooStream readBuffer size > 100000 ifTrue: [^nil].
	self readPosition - startPosition > 100000 
		ifTrue: [aSwazooStream enlargeReadBuffer: 1000000]
! !

!HTTPPost methodsFor: 'private-parsing' stamp: 'PaoloBonzini 10/4/2009 10:39'!
multipartDataFrom: aSwazooStream
	"read all mime parts and put them in postData" 
	"read directly from stream, without intermediate buffers"
	| contentTypeField boundary part |
	contentTypeField := self headers fieldOfClass: ContentTypeField
				ifNone: [^aSwazooStream nextBytes: self contentLength]. "just skip"
	boundary := contentTypeField transferCodings at: 'boundary'
				ifAbsent: [^aSwazooStream nextBytes: self contentLength]. "just skip"
	self skipMimePreambleAndBoundary: boundary from: aSwazooStream. 
	part := #something. 
	[part notNil] whileTrue: 
		[part := self partFromStream: aSwazooStream boundary: boundary.
		part notNil ifTrue: [self postDataAt: part key put: part value]].
	self skipMimeEpilogueFrom: aSwazooStream. "all to the end  as defined by contentLegth"
	aSwazooStream nilReadBuffer.! !

!HTTPPost methodsFor: 'private-parsing' stamp: 'PaoloBonzini 10/4/2009 11:13'!
readEntityFrom: aSwazooStream datum: aDatum boundary: aBoundaryBytes
	"read one entity from a stream and put into datum. Stream it if streamed. Also call a block 
	(if any) just before start of streaming, with a datum as parameter. This block can then set 
	a write stream in datum (for instance open a output file and stream on it)"
	| outStream |
	aDatum writeBlock notNil 
		ifTrue: [aDatum writeBlock value: aDatum]. "this should set writeStream if not already!!" 
	outStream := (aDatum isStreamed and: [aDatum writeStream notNil])
		ifTrue: [aDatum writeStream] ifFalse: [WriteStream on: ByteArray new].
	self blockStreamingFrom: aSwazooStream to: outStream until: ('--', aBoundaryBytes). "efficient streaming"
	aDatum isStreamed not ifTrue: "otherwise entity is already streamed to the output"
		[aDatum value: outStream contents asString].
	^aDatum! !

!HTTPPost methodsFor: 'parsing' stamp: 'PaoloBonzini 10/2/2009 18:57'!
readFrom: aSwazooStream 
	super readFrom: aSwazooStream.
	self initPostDataFor: aSwazooStream.
"	self parsePostDataFrom: aSwazooStream." "defered until first access of postData!! "! !

!HeaderField class methodsFor: 'instance creation' stamp: 'PaoloBonzini 10/2/2009 18:39'!
fromLine: aString
	"for testing only"
	| sourceStream fieldName fieldValue |
	sourceStream := ReadStream on: aString.
	fieldName := sourceStream upTo: $:.
	fieldValue := sourceStream upToEnd.
	^self name: fieldName value: fieldValue

! !

!HeaderField class methodsFor: 'private' stamp: 'PaoloBonzini 10/2/2009 18:40'!
name: fieldName value: fieldValue
	| upName trimValue fieldClass |
	upName := (HTTPString trimBlanksFrom: fieldName) asUppercase.
	fieldClass := self classForFieldName: upName.
	trimValue := HTTPString trimBlanksFrom: fieldValue.
	^fieldClass newForFieldName: upName withValueFrom: trimValue
! !

!HTTPResponse class methodsFor: 'accessing' stamp: 'PaoloBonzini 10/2/2009 18:45'!
statusTextForCode: aNumber
	^StatusCodes at: aNumber ifAbsent: [
		"if some new status codes was added later"
		self initialize.
		StatusCodes at: aNumber ifAbsent: ['']]
! !

!HTTPResponse methodsFor: 'accessing-headers' stamp: 'PaoloBonzini 10/2/2009 18:45'!
contentLength
	^self headers
		fieldNamed: 'Content-length'
		ifNone: [
			| field |
			field := HTTPContentLengthField new contentLength: self contentSize.
			self headers addField: field.
			field ]
! !

TestCase subclass: #SwazooBoundaryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!SwazooBoundaryTest methodsFor: 'testing-mime boundary' stamp: 'PaoloBonzini 10/4/2009 11:51'!
testBoundaryFull
	| boundary stream |
	boundary := '--boundary--' asByteArray.
	stream := SwazooStream on: 'just--boundary--something'.	"full boundary"
	self assert: (stream signsOfBoundary: boundary) = boundary size
! !

!SwazooBoundaryTest methodsFor: 'testing-mime boundary' stamp: 'PaoloBonzini 10/4/2009 11:51'!
testBoundaryMixed
	| boundary stream |
	boundary := '--boundary--' asByteArray.
	stream := SwazooStream on: 'yes,--just--boundary--something'.	"partial, later full boundary"
	self assert: (stream signsOfBoundary: boundary) = boundary size
! !

!SwazooBoundaryTest methodsFor: 'testing-mime boundary' stamp: 'PaoloBonzini 10/4/2009 11:51'!
testBoundaryOnEdge
	"part of boundary at the end of this stream, remaining probably in the next"
	| boundary stream |
	boundary := '--boundary--' asByteArray.
	stream := SwazooStream on: 'just something-'.	"just first char of boundary"
	self assert: (stream signsOfBoundary: boundary) = 1.
	stream := SwazooStream on: 'just something--'.	"two chars"
	self assert: (stream signsOfBoundary: boundary) = 2.
	stream := SwazooStream on: 'just something--bound'.	"half"
	self assert: (stream signsOfBoundary: boundary) = 7.
	stream := SwazooStream on: 'just something--boundary--'.	"full boundary at the edge"
	self assert: (stream signsOfBoundary: boundary) = boundary size
! !

!SwazooBoundaryTest methodsFor: 'testing-mime boundary' stamp: 'PaoloBonzini 10/4/2009 11:51'!
testBoundaryOnEdgeMixed
	"signs of boundary in the middle part at the end of this buffer, remaining probably in the next"
	| boundary stream |
	boundary := '--boundary--' asByteArray.
	stream := SwazooStream on: 'just-something-'.	"sign in the middle, one char at the end"
	self assert: (stream signsOfBoundary: boundary) = 1.
	stream := SwazooStream on: 'just-something--'.	"two chars"
	self assert: (stream signsOfBoundary: boundary) = 2.
	stream := SwazooStream on: 'just-so--mething--bound'.	"even more mixed case"
	self assert: (stream signsOfBoundary: boundary) = 7
! !

!SwazooBoundaryTest methodsFor: 'testing-mime boundary' stamp: 'PaoloBonzini 10/4/2009 11:51'!
testBoundarySimple
	| boundary stream |
	boundary := '--boundary--' asByteArray.
	stream := SwazooStream on: 'just something'.	"no boundary"
	self assert: (stream signsOfBoundary: boundary) = 0.
	stream := SwazooStream on: 'just-something'.	"sign of boundary"
	self assert: (stream signsOfBoundary: boundary) = 0.
	stream := SwazooStream on: 'just--something'.	"more sign of boundary"
	self assert: (stream signsOfBoundary: boundary) = 0.
	stream := SwazooStream on: 'just--boundary--something'.	"full boundary"
	self assert: (stream signsOfBoundary: boundary) = boundary size
! !

!SwazooBoundaryTest methodsFor: 'testing-mime boundary' stamp: 'PaoloBonzini 10/4/2009 11:51'!
testIndexOfBoundary
	"index of start of boundary in buffer, both full or partial at the edge/end of buffer"
	| boundary stream |
	boundary := '--boundary--' asByteArray.
	stream := SwazooStream on: 'just something'.	"no boundary"
	self assert: (stream indexOfBoundary: boundary) = 0.
	stream := SwazooStream on: 'just--boundary--something-'.	"full boundary"
	self assert: (stream indexOfBoundary: boundary) = 5.
	stream := SwazooStream on: 'just something--boun'.	"partial boundary at the edge"
	self assert: (stream indexOfBoundary: boundary) = 15.
	stream := SwazooStream on: 'just something-'.	"partial boundary, one char only"
	self assert: (stream indexOfBoundary: boundary) = 15.
	stream := SwazooStream on: 'just-som--ething--boun'.	"mixed case with partial at the edge"
	self assert: (stream indexOfBoundary: boundary) = 17
! !

!HeaderFieldTest methodsFor: 'testing' stamp: 'PaoloBonzini 10/4/2009 10:29'!
testContentTypeMultiple
	"   HTTP/1.1 header field values can be folded onto multiple lines if the
   continuation line begins with a space or horizontal tab. All linear
   white space, including folding, has the same semantics as SP. A
   recipient MAY replace any linear white space with a single SP before
   interpreting the field value or forwarding the message downstream.

       LWS            = [CRLF] 1*( SP | HT )"

	| requestStream request field |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET / HTTP/1.1';
		nextPutLine: 'Host: 127.0.0.1';
		nextPutLine: 'Content-Type: text/html; ';
		nextPutLine: ' charset=iso-8859-1';
		crlf.
	request := HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents).
	field := request headers fieldNamed: 'content-type'.
	self assert: field name = 'Content-Type'.
	self assert: field mediaType = 'text/html'.
	self assert: (field transferCodings at: 'charset') = 'iso-8859-1'! !

!SwazooStreamTest methodsFor: 'testing-lines' stamp: 'PaoloBonzini 10/4/2009 10:30'!
testLinesWithDoubleCRLF
	| ws rs comparisonString |
	comparisonString := 'abcd'.
	ws := SwazooStream on: String new.
	ws nextPutAll: comparisonString.
	self crlfOn: ws.
	self crlfOn: ws.
	rs := SwazooStream on: ws writeBufferContents.
	self assert: rs nextLine = comparisonString.
	self assert: rs nextLine = ''! !

!SwazooStreamTest methodsFor: 'testing-lines' stamp: 'PaoloBonzini 10/4/2009 10:30'!
testSingleLineWithCR
	| ws rs comparisonString errored |
	comparisonString := 'abcd' , (String with: Character cr) , 'efg'.
	ws := SwazooStream on: String new.
	ws nextPutAll: comparisonString.
	ws nextPut: Character cr.
	rs := SwazooStream on: ws writeBufferContents.
	errored := false.
	SpExceptionContext 
		for: [rs nextLine]
		on: SpError
		do: [:ex | errored := true].
	self assert: errored! !

!SwazooStreamTest methodsFor: 'testing-lines' stamp: 'PaoloBonzini 10/4/2009 10:30'!
testSingleLineWithCRLF
	| ws rs comparisonString |
	comparisonString := 'abcd'.
	ws := SwazooStream on: String new.
	ws nextPutAll: comparisonString.
	self crlfOn: ws.
	rs := SwazooStream on: ws writeBufferContents.
	self assert: rs nextLine = comparisonString! !

Object subclass: #AbstractHTTPServer
	instanceVariableNames: 'sites socket loop isMultiThreading'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

!AbstractHTTPServer methodsFor: 'private' stamp: 'PaoloBonzini 10/3/2009 01:25'!
acceptConnection
	"^self
I accept the next inbound TCP/IP connection.  The operating system libraries queue these up for me, so I can just handle one at a time.  I create an HTTPConnection instance to actually handle the interaction with the client - if I am in single threaded mode, the connection will completely handle the request before returning control to me, but in multi-threaded mode the connection forks the work into a sepparate thread in this image and control is immediately returned to me (the application programmer must worry about thread safety in this case."

	(SwazooServer singleton connectionPool acceptConnectionForServer: self)
		interact: self isMultiThreading ! !

!AbstractHTTPServer methodsFor: 'private' stamp: 'PaoloBonzini 10/3/2009 01:26'!
acceptConnectionFromSocket
	| connection |
	connection := HTTPConnection socket: self socket accept.
	connection server: self.
	^ connection! !

!AbstractHTTPServer methodsFor: 'sites' stamp: 'PaoloBonzini 10/3/2009 01:26'!
addSite: aSite
	(self sites includesResource: aSite) ifFalse: [^self sites addResource: aSite]! !

!AbstractHTTPServer methodsFor: 'serving' stamp: 'PaoloBonzini 10/3/2009 01:26'!
answerTo: aRequest 
	| response |
	response := URIResolution resolveRequest: aRequest startingAt: self sites.
	^response isNil
		ifTrue: [HTTPResponse notFound]
		ifFalse: [response]! !

!AbstractHTTPServer methodsFor: 'start/stop' stamp: 'PaoloBonzini 10/3/2009 01:25'!
createSocket
	self subclassResponsibility! !

!AbstractHTTPServer methodsFor: 'sites' stamp: 'PaoloBonzini 10/3/2009 01:26'!
hasNoSites
	^self sites hasNoResources! !

!AbstractHTTPServer methodsFor: 'private-initialize' stamp: 'PaoloBonzini 10/3/2009 01:26'!
initSites
	sites := ServerRootComposite new! !

!AbstractHTTPServer methodsFor: 'private-initialize' stamp: 'PaoloBonzini 10/3/2009 01:26'!
initialize
	self initSites! !

!AbstractHTTPServer methodsFor: 'multithreading' stamp: 'PaoloBonzini 10/3/2009 01:26'!
isMultiThreading
	"^a Boolean
I return true if each inbound HTTP connection will be handled in its own thread.  See the senders of this message to see where that is important.  Note that the default mode is mult-threaded because this is how Swazoo has worked so far.  This is tricky for the application programmer, though, as they must ensure that they work in a thread safe way (e.g. avoid the many threads updating the same object).  For those deploying to GemStone, you wil find things much easier if you do *not* run multithreaded, but rather run many gems each with a single-threaded Swazoo instance (and your app logic) in each.  Also in GemStone, run the main loop in the foreground, c.f. >>mainLoopInForeground"

	isMultiThreading isNil ifTrue: [self setMultiThreading].
	^isMultiThreading! !

!AbstractHTTPServer methodsFor: 'testing' stamp: 'PaoloBonzini 10/3/2009 01:26'!
isServing
	^self loop notNil! !

!AbstractHTTPServer methodsFor: 'private' stamp: 'PaoloBonzini 10/3/2009 01:27'!
loop
	^loop! !

!AbstractHTTPServer methodsFor: 'private' stamp: 'PaoloBonzini 10/3/2009 01:27'!
loop: aProcess
	loop := aProcess! !

!AbstractHTTPServer methodsFor: 'private' stamp: 'PaoloBonzini 10/3/2009 01:27'!
removeConnection: aConnection 
	SwazooServer singleton connectionPool removeConnection: aConnection! !

!AbstractHTTPServer methodsFor: 'sites' stamp: 'PaoloBonzini 10/3/2009 01:27'!
removeSite: aSite 
	^self sites removeResource: aSite! !

!AbstractHTTPServer methodsFor: 'serving' stamp: 'PaoloBonzini 10/3/2009 01:25'!
requestReaderClass
	self subclassResponsibility! !

!AbstractHTTPServer methodsFor: 'serving' stamp: 'PaoloBonzini 10/3/2009 01:25'!
responsePrinterClass
	self subclassResponsibility! !

!AbstractHTTPServer methodsFor: 'start/stop' stamp: 'PaoloBonzini 10/3/2009 01:27'!
restart
	"usefull after image startup, when socket is probably not valid anymore"
	self stop.
	self start.! !

!AbstractHTTPServer methodsFor: 'multithreading' stamp: 'PaoloBonzini 10/3/2009 01:27'!
setMultiThreading
	"^self
I record that this HTTP server is to operate in a multi-threaded mode.  c.f. isMultiThreading"

	isMultiThreading := true.
	^self! !

!AbstractHTTPServer methodsFor: 'multithreading' stamp: 'PaoloBonzini 10/3/2009 01:27'!
setSingleThreading
	"^self
I record that this HTTP server is to operate in a single-threaded mode.  c.f. isMultiThreading"

	isMultiThreading := false.
	^self! !

!AbstractHTTPServer methodsFor: 'private' stamp: 'PaoloBonzini 10/3/2009 01:27'!
sites
	^sites! !

!AbstractHTTPServer methodsFor: 'private' stamp: 'PaoloBonzini 10/3/2009 01:27'!
socket
	^socket! !

!AbstractHTTPServer methodsFor: 'private' stamp: 'PaoloBonzini 10/3/2009 01:27'!
socket: aSocket
	socket := aSocket! !

!AbstractHTTPServer methodsFor: 'start/stop' stamp: 'PaoloBonzini 10/3/2009 01:27'!
start
	self loop isNil ifTrue: 
		[ self socket: self createSocket.
		self loop: ([ [ self acceptConnection ] repeat ] forkAt: Processor userBackgroundPriority) ]! !

!AbstractHTTPServer methodsFor: 'start/stop' stamp: 'PaoloBonzini 10/3/2009 01:27'!
stop
	self loop isNil 
		ifFalse: 
			[SwazooServer singleton connectionPool connections
				do: [:each | each server == self ifTrue: [each close]].
			self loop terminate.
			self loop: nil.
			self socket close.
			self socket: nil]! !

AbstractHTTPServer subclass: #HTTPServer
	instanceVariableNames: 'ip port'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

!HTTPServer methodsFor: 'start/stop' stamp: 'PaoloBonzini 10/3/2009 01:22'!
createSocket
	^(self socketClass serverOnIP: self ipCorrected port: self port)
		listenFor: 50;
		yourself
! !

!HTTPServer methodsFor: 'serving' stamp: 'PaoloBonzini 10/2/2009 20:53'!
requestReaderClass
	^HTTPReader ! !

!HTTPServer methodsFor: 'serving' stamp: 'PaoloBonzini 10/2/2009 20:53'!
responsePrinterClass 
	^HTTPPrinter ! !

Object subclass: #AbstractSwazooSocket
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

!AbstractSwazooSocket methodsFor: 'server accessing' stamp: 'PaoloBonzini 10/3/2009 01:35'!
accept
	self subclassResponsibility! !

!AbstractSwazooSocket methodsFor: 'accessing' stamp: 'PaoloBonzini 10/3/2009 01:35'!
close
	self subclassResponsibility! !

!AbstractSwazooSocket methodsFor: 'testing' stamp: 'PaoloBonzini 10/3/2009 01:35'!
isActive
	self subclassResponsibility! !

!AbstractSwazooSocket methodsFor: 'server accessing' stamp: 'PaoloBonzini 10/3/2009 01:35'!
listenFor: anInteger
	self subclassResponsibility! !

!AbstractSwazooSocket methodsFor: 'accessing' stamp: 'PaoloBonzini 10/3/2009 01:35'!
localAddress
	self subclassResponsibility! !

!AbstractSwazooSocket methodsFor: 'accessing' stamp: 'PaoloBonzini 10/3/2009 01:35'!
read: anInteger
	self subclassResponsibility! !

!AbstractSwazooSocket methodsFor: 'accessing' stamp: 'PaoloBonzini 10/3/2009 01:35'!
readInto: aByteArray startingAt: start for: length
	self subclassResponsibility! !

!AbstractSwazooSocket methodsFor: 'accessing' stamp: 'PaoloBonzini 10/3/2009 01:35'!
remoteAddress
	self subclassResponsibility! !

!AbstractSwazooSocket methodsFor: 'private' stamp: 'PaoloBonzini 10/3/2009 01:36'!
stream
	^SwazooStream socket: self! !

!AbstractSwazooSocket methodsFor: 'accessing' stamp: 'PaoloBonzini 10/3/2009 01:35'!
write: aByteArray
	self subclassResponsibility! !

!AbstractSwazooSocket methodsFor: 'accessing' stamp: 'PaoloBonzini 10/3/2009 01:35'!
writeFrom: aByteArray startingAt: start for: length
	self subclassResponsibility! !

AbstractSwazooSocket subclass: #SwazooSocket
	instanceVariableNames: 'accessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

Object subclass: #HTTPConnection
	instanceVariableNames: 'stream loop id server task mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

!HTTPConnection class methodsFor: 'instance creation' stamp: 'PaoloBonzini 10/2/2009 21:49'!
socket: aSocket 
	^self new initializeStream: aSocket stream! !

!HTTPConnection methodsFor: 'testing' stamp: 'PaoloBonzini 10/2/2009 21:51'!
anyDataReady 
	"wait for data and if anything is read, proeed"
	| theStream |
	theStream := self stream.
	^self isOpen and: [theStream anyDataReady]! !

!HTTPConnection methodsFor: 'serving' stamp: 'PaoloBonzini 10/2/2009 21:51'!
getAndDispatchRequest
	"^self
	The HTTPRequest is read from my socket stream. I then pass
	this request to my server
	to get a response."
	self task: (SwazooTask newOn: self).
	self readRequestFor: self task.
	mutex critical: [
		self isOpen ifFalse: [^ self].
		self produceResponseFor: self task].
	self task request wantsConnectionClose ifTrue: [self close].
	self task request isHttp10 ifTrue: [self close]! !

!HTTPConnection methodsFor: 'accessing' stamp: 'PaoloBonzini 10/2/2009 21:56'!
id
	^id
! !

!HTTPConnection methodsFor: 'accessing' stamp: 'PaoloBonzini 10/2/2009 21:57'!
id: anInteger
	id := anInteger! !

!HTTPConnection methodsFor: 'initialize' stamp: 'PaoloBonzini 10/2/2009 21:57'!
initializeStream: aSwazooStream 
	mutex := Semaphore forMutualExclusion.
	stream := aSwazooStream! !

!HTTPConnection methodsFor: 'serving' stamp: 'PaoloBonzini 10/2/2009 22:29'!
interact: multithreaded
	"longer description is below method"
	| interactionBlock |
	interactionBlock := 
		[[	[[self anyDataReady] whileTrue: 
				[self getAndDispatchRequest.
				Processor yield]]
					ensure: [self close]]
			on: Error
			do: [:ex |
				(Delay forMilliseconds: 50) wait. "to finish sending, if any"
				self close]].
	multithreaded
		ifTrue: [self loop: (interactionBlock forkAt: Processor userBackgroundPriority)]
		ifFalse: [interactionBlock value].
	^self

"I represent a specifc connection with an HTTP client (a browser, probably) over which will come an HTTP request.  Here, I fork the handling of the request so that the current thread (which is most likely the HTTP server main loop) can carry on with the next request.  This means that more than one request may being handled in the image at a time, and that means that the application developer must worry about thread safety - e.g the problem of a given business object being updated by more than one HTTP request thread.
For a GemStone implementation of Swazoo, one may want only one request is handled at a time, multi-threadedness being handled by having multiple gems.  This is a nice option because the application developer does not have to worry about thread safety in this case - GemStone handles the hard stuff.
*And* the thing called a loop that was in this method was no such thing.  In all circumstances, >>getAndDispatchRequest handles exactly one requst and then closes the socket!! (very non-HTTP1.1).  Anyway, for now I'm just going to make that explicit.  This needs to be re-visited to support HTTP 1.1."! !

!HTTPConnection methodsFor: 'testing' stamp: 'PaoloBonzini 10/2/2009 21:56'!
keepAliveTimeout
	| seconds task request |
	(task := self task) isNil ifTrue: [^ false].
	(request := task request) isNil ifTrue: [^ false].
	request isKeepAlive ifFalse: [^ false].
	seconds := request keepAlive notNil
				ifTrue: [request keepAlive asInteger]
				ifFalse: [20].	"probably enough?"
	^ SpTimestamp now asSeconds - request timestamp asSeconds >= seconds! !

!HTTPConnection methodsFor: 'serving-responses' stamp: 'PaoloBonzini 10/4/2009 10:32'!
nextPutError: aResponse 
	aResponse informConnectionClose.
	self responsePrinterClass writeResponse: aResponse to: self stream.
! !

!HTTPConnection methodsFor: 'serving-responses' stamp: 'PaoloBonzini 10/4/2009 10:32'!
nextPutResponse: aMessage toRequest: aRequest 
	aRequest isHead 
		ifTrue: [self responsePrinterClass writeHeadersFor: aMessage to: self stream]
		ifFalse: [self responsePrinterClass writeResponse: aMessage to: self stream].! !

!HTTPConnection methodsFor: 'serving' stamp: 'PaoloBonzini 10/2/2009 18:37'!
readRequestFor: aSwazooTask 
	"I read the next request from my socket and add it to aSwazooTask.  If I have any 
	problems and need to force a bad request (400) response, I add this response to aSwazooTask."
	| request |
	SpExceptionContext for: 
		[request := self requestReaderClass readFrom: self stream.
		request uri port: self server port.
		(request httpVersion last = 1 
			and: [(request headers includesFieldOfClass: HTTPHostField) not]) 
				ifTrue: [aSwazooTask response: HTTPResponse badRequest].
		[request peer: self stream socket remoteAddress]
			on: Error do: [:ex | "do nothing for now"].
		request
			ip: self stream socket localAddress;
			setTimestamp.
		aSwazooTask request: request]
	on: SpError, HTTPException
	do: [:ex | 
		aSwazooTask response: HTTPResponse badRequest.
		self nextPutError: aSwazooTask response.
		self close].! !

!HTTPConnection methodsFor: 'serving' stamp: 'PaoloBonzini 10/2/2009 18:37'!
requestReaderClass
	^server requestReaderClass! !

!HTTPConnection methodsFor: 'serving' stamp: 'PaoloBonzini 10/2/2009 18:37'!
responsePrinterClass 
	^server responsePrinterClass 
! !

!HTTPConnection methodsFor: 'testing' stamp: 'PaoloBonzini 10/2/2009 21:49'!
whenNotServingDo: aBlock
	mutex critical: aBlock! !

Object subclass: #HTTPConnectionPool
	instanceVariableNames: 'reapers pool prev next first last firstFree mutex reapDone'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

!HTTPConnectionPool commentStamp: '<historical>' prior: 0!
HTTPConnectionPool is a helper class that manages all the
connections of a SwazooServer.  The number of available connections
is limited, so the pool is always kept sorted by first request and
more ancient connections are dropped if there is a request from
somewhere else.

In addition, this class implements a backoff policy that should help in
case limited resources prevent acceptance of new connections.
!

!HTTPConnectionPool classSide methodsFor: 'instance creation' stamp: ''!
new: size
	^self new initialize: size
! !

!HTTPConnectionPool methodsFor: 'serving' stamp: ''!
acceptConnectionForServer: aServer
	"Ask aServer for an HTTPConnection object, put it into the pool,
	 and return it."
	| connection time |
	time := 10.
	[connection := SpExceptionContext 
				for: [aServer acceptConnectionFromSocket]
				on: SpError
				do: [:ex |
					Transcript
						show: 'Socket accept error: ' , ex errorString;
						cr.
					ex return].

	connection isNil] whileTrue: [
		 "Trying immediately would just cause another failure.
		  Try freeing a connection's resources first, otherwise wait
		  for connections to finish in 10, 30, 70, 150, 250 ms."
		 self tryToReapConnection ifFalse: [
			(Delay forMilliseconds: time) wait.
			time := (time + 5) * 2 min: 250]].

	^self addConnection: connection
! !

!HTTPConnectionPool methodsFor: 'private' stamp: ''!
addConnection: aConnection
	mutex critical: [
		firstFree = 0 ifTrue: [self reapConnection].
		aConnection id: firstFree.

		"Add to the list and update the free list."
		pool at: firstFree put: aConnection.
		prev at: firstFree put: last.
		last = 0
			ifTrue: [ first := last := firstFree ]
			ifFalse: [ last := next at: last put: firstFree ].

		firstFree := next at: firstFree.
		next at: last put: 0].

	^aConnection
! !

!HTTPConnectionPool methodsFor: 'accessing' stamp: ''!
connections
	| oc |
	oc := OrderedCollection new.
	mutex critical: [
		self walk: [:each | oc add: each]].
	^oc
! !

!HTTPConnectionPool methodsFor: 'private-initialize' stamp: ''!
initialize: size
	"Free list."
	firstFree := size.
	next := (0 to: size - 1) asArray.

	"Actual connection pool."
	first := last := 0.
	pool := Array new: size.
	prev := Array new: size.

	mutex := Semaphore forMutualExclusion.
	reapers := (1 to: size) collect: [ :i | self reaperForIndex: i ].
! !

!HTTPConnectionPool methodsFor: 'reaping' stamp: ''!
reapConnection
	"Kill one dormient connection, possibly sleeping until one is found."
	self reapConnections: true
! !

!HTTPConnectionPool methodsFor: 'reaping' stamp: ''!
reapConnections: block
	"Must be called within mutex."
	| i sem prev |

	"First the easy case, see if some Keep-Alive connection has
	 timed out."
	self walk: [ :conn |
		conn keepAliveTimeout ifTrue: [conn close. ^self]].

	"Else wake up the reaper processes.  They will find the first
	 connection that finishes serving a response and forcibly close
	 it."
	i := first.
	sem := reapDone := Semaphore new.
	[reapDone notNil and: [i > 0]] whileTrue: [
		prev := i.
		i := next at: i.
		mutex signal.
		self tryToReap: prev.
		mutex wait].

	"Wait for the processes to do their job if called because of
	 excessive loads."
	block ifTrue: [sem wait].
	reapDone := nil.
! !

!HTTPConnectionPool methodsFor: 'private-initialize' stamp: ''!
reaperForIndex: i
	| block s |
	s := Semaphore new.

	"We need a separate process because Smalltalk Semaphores
	 do not portably provide a 'try to lock' method.  We store
	 a semaphore in an array and use the semaphore to wake up
	 the process."
	block := [
		| sem doIt conn |
		[s wait.
		(sem := reapDone) notNil ifTrue: [
			conn := pool at: i.
			conn isNil
				ifTrue: [
					reapDone := nil.
					sem signal]
				ifFalse: [
					conn whenNotServingDo: [
						"Run under mutex to ensure only one connection
						 is reaped."
						mutex critical: [
							doIt := reapDone == sem.
							doIt ifTrue: [reapDone := nil]]].

					doIt ifTrue: [
						conn close.
						sem signal]]]]
			repeat].

	block forkAt: Processor userInterruptPriority.
	^s
! !

!HTTPConnectionPool methodsFor: 'accessing' stamp: ''!
removeConnection: aConnection
	| index |
	index := aConnection id.
	aConnection id: 0.
	index = 0 ifTrue: [ ^self ].
	mutex critical: [
		"Remove from the list..."
		| itsPrev itsNext |
		itsPrev := prev at: index.
		itsNext := next at: index.
		pool at: index put: nil.
		itsNext = 0
			ifTrue: [ last := itsPrev ]
			ifFalse: [ prev at: itsNext put: itsPrev ].
		itsPrev = 0
			ifTrue: [ first := itsNext ]
			ifFalse: [ next at: itsPrev put: itsNext ].

		"... and put back into the free list."
		next at: index put: firstFree.
		firstFree := index].
! !

!HTTPConnectionPool methodsFor: 'accessing' stamp: ''!
size
	^pool size
! !

!HTTPConnectionPool methodsFor: 'reaping' stamp: ''!
tryToReap: i
	"Start a process that will wait for connection i to finish
	 serving a response and, if no other connection has been reaped
	 yet, will forcibly close connection i."
	"Must be called within mutex."
	(reapers at: i) signal.
! !

!HTTPConnectionPool methodsFor: 'reaping' stamp: ''!
tryToReapConnection
	"Look for dormient connections and kill one of them.  Return whether
	 one was found."
	^mutex critical: [
		self reapConnections: false.
		firstFree > 0]
! !

!HTTPConnectionPool methodsFor: 'private' stamp: ''!
walk: aBlock
	"Must be called within mutex."
	| i |
	i := first.
	[i = 0] whileFalse: [
		aBlock value: (pool at: i). 
		i := next at: i]
! !

Object subclass: #SwazooRequestReader
	instanceVariableNames: 'request stream requestLine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

SwazooRequestReader subclass: #HTTPReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

!HTTPReader methodsFor: 'parsing' stamp: ''!
parseHTTPVersion
	| major minor |
	self skipSpaces.
	stream upTo: $/ asInteger.
	major := (stream upTo: $. asInteger) asString asNumber.
	minor := (stream upTo: Character cr asInteger) asString asNumber.
	self httpVersion: (Array with: major with: minor).
	stream next.
! !

!HTTPReader methodsFor: 'parsing' stamp: ''!
parseURI
	self skipSpaces.
	self requestURI:
		(SwazooURI
			fromString: (stream upTo: Character space asInteger) asString).
	^self
! !

!HTTPReader methodsFor: 'parsing' stamp: ''!
readHeaderFieldFrom: aString
	| sourceStream fieldName fieldValue |
	sourceStream := ReadStream on: aString.
	fieldName := sourceStream upTo: $:.
	fieldValue := sourceStream upToEnd.
	^HeaderField name: fieldName value: fieldValue
! !

!HTTPReader methodsFor: 'parsing' stamp: ''!
readHeaders
	| nextLine field header |

	[nextLine := stream nextUnfoldedLine.
	nextLine isEmpty] whileFalse: [
	request headers addField: (self readHeaderFieldFrom: nextLine)].
	^self
! !

!HTTPReader methodsFor: 'parsing' stamp: ''!
readRequestLine
	self skipLeadingBlankLines.
	self method: (stream upTo: Character space asInteger) asString.
	self parseURI.
	self parseHTTPVersion.
! !

!HTTPReader methodsFor: 'parsing' stamp: ''!
skipLeadingBlankLines
	"^self
	 RFC 2616:
	 In the interest of robustness, servers SHOULD ignore any empty
	 line(s) received where a Request-Line is expected. In other words, if
	 the server is reading the protocol stream at the beginning of a
	 message and receives a CRLF first, it should ignore the CRLF."
	[stream peek == Character cr asInteger] whileTrue:
			[((stream next: 2) at: 2) == Character lf asInteger
				ifFalse: [SwazooHTTPParseError raiseSignal: 'CR without LF']].
	^self
! !

!HTTPReader methodsFor: 'parsing' stamp: ''!
skipSpaces
	[stream peek = Character space] whileTrue: [stream next].
	^self
! !

!SwazooRequestReader classSide methodsFor: 'parsing' stamp: ''!
readFrom: aSwazooStream
	| reader |
	reader := self new.
	reader initializeStream: aSwazooStream.
	reader readRequest.
	^reader request
! !

!SwazooRequestReader methodsFor: 'accessing-request line' stamp: ''!
httpVersion
	^requestLine httpVersion
! !

!SwazooRequestReader methodsFor: 'accessing-request line' stamp: ''!
httpVersion: aString
	requestLine httpVersion: aString
! !

!SwazooRequestReader methodsFor: 'initialize' stamp: ''!
initializeStream: aSwazooStream
	stream := aSwazooStream.
	requestLine := HTTPRequestLine new.
! !

!SwazooRequestReader methodsFor: 'accessing-request line' stamp: ''!
method
	^requestLine method
! !

!SwazooRequestReader methodsFor: 'accessing-request line' stamp: ''!
method: aString
	requestLine method: aString
! !

!SwazooRequestReader methodsFor: 'parsing' stamp: ''!
readBody
	self request readFrom: stream
! !

!SwazooRequestReader methodsFor: 'abstract-parsing' stamp: ''!
readHeaders
	self subclassResponsibility
! !

!SwazooRequestReader methodsFor: 'parsing' stamp: ''!
readRequest
	self readRequestLine.
	request := (HTTPRequest classFor: self method) new.
	request requestLine: requestLine.
	self readHeaders.
	self request setTimestamp.
	self readBody
! !

!SwazooRequestReader methodsFor: 'abstract-parsing' stamp: ''!
readRequestLine
	self subclassResponsibility
! !

!SwazooRequestReader methodsFor: 'accessing' stamp: ''!
request
	^request
! !

!SwazooRequestReader methodsFor: 'accessing-request line' stamp: ''!
requestURI
	^requestLine requestURI
! !

!SwazooRequestReader methodsFor: 'accessing-request line' stamp: ''!
requestURI: aString
	requestLine requestURI: aString
! !

!SwazooRequestReader methodsFor: 'accessing' stamp: ''!
stream
	^stream
! !

Object subclass: #SwazooResponsePrinter
	instanceVariableNames: 'stream response'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

SwazooResponsePrinter subclass: #HTTPPrinter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

!HTTPPrinter methodsFor: 'private-sending' stamp: ''!
crlf
	stream
		nextPut: Character cr;
		nextPut: Character lf
! !

!HTTPPrinter methodsFor: 'private-sending' stamp: ''!
endHeader
	self crlf
! !

!HTTPPrinter methodsFor: 'private-sending' stamp: ''!
printChunkedTransferEncoding
	stream nextPutAll: 'Transfer-Encoding: chunked'.
	self crlf
! !

!HTTPPrinter methodsFor: 'private-sending' stamp: ''!
printHeader: aField
	stream
		nextPutAll: aField name;
		nextPutAll: ': '.
	aField valuesAsStringOn: stream.
	self crlf
! !

!HTTPPrinter methodsFor: 'private-sending' stamp: ''!
printStatus
	| version |
	version := (response task isNil 
			or: [response task request isNil or: [response task request isHttp11]]) 
				ifTrue: ['HTTP/1.1 ']
				ifFalse: ['HTTP/1.0 '].
	stream
		nextPutAll: version;
		print: response code;
		space;
		nextPutAll: response codeText.
	self crlf
! !

!SwazooResponsePrinter classSide methodsFor: 'private-sending' stamp: 'PaoloBonzini 10/4/2009 10:33'!
writeHeadersFor: aResponse to: aSwazooStream
	aSwazooStream isNil ifTrue: [^self].
	self new
		response: aResponse;
		stream: aSwazooStream;
		writeHeader;
		closeResponse 
! !

!SwazooResponsePrinter classSide methodsFor: 'private-sending' stamp: 'PaoloBonzini 10/4/2009 10:33'!
writeResponse: aResponse for: aRequest to: aSwazooStream
	aSwazooStream isNil ifTrue: [^self].
	self new
		response: aResponse;
		stream: aSwazooStream;
		writeResponseTo: aRequest;
		closeResponse ! !

!SwazooResponsePrinter classSide methodsFor: 'private-sending' stamp: 'PaoloBonzini 10/4/2009 10:33'!
writeResponse: aResponse to: aSwazooStream
	aSwazooStream isNil ifTrue: [^self].
	self new
		response: aResponse;
		stream: aSwazooStream;
		writeResponseTo: nil;
		closeResponse 
! !

!SwazooResponsePrinter methodsFor: 'abstract-sending' stamp: ''!
endHeader
	self subclassResponsibility
! !

!SwazooResponsePrinter methodsFor: 'abstract-sending' stamp: ''!
printChunkedTransferEncoding
	self subclassResponsibility
! !

!SwazooResponsePrinter methodsFor: 'sending' stamp: ''!
printContentLength
	"it is also added to headers. It is added so late because to be printed last,
	 just before body starts"
	self printHeader: response contentLength
! !

!SwazooResponsePrinter methodsFor: 'abstract-sending' stamp: ''!
printHeader: aField
	self subclassResponsibility
! !

!SwazooResponsePrinter methodsFor: 'sending' stamp: ''!
printHeaders
	"^self
	 Write the headers (key-value pairs) to aStream.  The key
	 must be a String."
	response headers fields do: 
		[:aField | self printHeader: aField]
! !

!SwazooResponsePrinter methodsFor: 'abstract-sending' stamp: ''!
printStatus
	self subclassResponsibility
! !

!SwazooResponsePrinter methodsFor: 'accessing' stamp: ''!
response
	^response
! !

!SwazooResponsePrinter methodsFor: 'accessing' stamp: ''!
response: aResponse
	response := aResponse
! !

!SwazooResponsePrinter methodsFor: 'accessing' stamp: ''!
stream
	^stream
! !

!SwazooResponsePrinter methodsFor: 'accessing' stamp: ''!
stream: aSwazooStream
	stream := aSwazooStream
! !

!SwazooResponsePrinter methodsFor: 'sending' stamp: ''!
writeHeader
	self printStatus.
	self printHeaders.
	(response isStreamed and: [response shouldBeChunked]) 
		ifTrue: [self printChunkedTransferEncoding]
		ifFalse: [self printContentLength].
	self endHeader
! !

!SwazooResponsePrinter methodsFor: 'sending' stamp: ''!
writeResponseTo: aRequest
	stream isNil ifTrue: [^self].
	self writeHeader.
	(aRequest isNil or: [aRequest isHead not]) 
		ifTrue: [response printEntityOn: self stream].
	stream closeResponse
! !

Object subclass: #SwazooServer
	instanceVariableNames: 'connectionPool sites servers watchdog'
	classVariableNames: 'Singleton'
	poolDictionaries: ''
	category: 'Swazoo-Core'!

!SwazooServer commentStamp: 'mivsek 12/31/2007 16:37' prior: 0!
SwazooServer is where all begins in Swazoo!!
SwazooServer singleton : return one and only one server which holds the Sites. Also used to start and stop all sites ato once, to add new sited etc. When running, a collection of HTTPServers is also stored in SwazooServer singleton.

SwazooServer demoStart  will create and run a demo site on http://localhost:8888 which 
                              returns a web page with 'Hello World!!'!

!SwazooServer class methodsFor: 'private' stamp: 'PaoloBonzini 10/2/2009 21:46'!
defaultConnectionPoolSize
	^15! !

!SwazooServer class methodsFor: 'private' stamp: 'PaoloBonzini 10/2/2009 21:46'!
initSingleton
	Singleton := self basicNew initialize! !

!SwazooServer methodsFor: 'accessing' stamp: 'PaoloBonzini 10/2/2009 22:34'!
connectionPool
	connectionPool isNil
		ifTrue: [self connectionPoolSize: self class defaultConnectionPoolSize].
	^connectionPool! !

!SwazooServer methodsFor: 'accessing' stamp: 'PaoloBonzini 10/2/2009 21:47'!
connectionPoolSize
	^connectionPool size! !

!SwazooServer methodsFor: 'accessing' stamp: 'PaoloBonzini 10/2/2009 21:47'!
connectionPoolSize: anInteger 
	| active |
	active := self servers isNil
				ifTrue: [#()]
				ifFalse: [self servers copy].
	active do: [:each | each stop].
	connectionPool := HTTPConnectionPool new: anInteger.
	active do: [:each | each start]! !

!SwazooServer methodsFor: 'initialize-release' stamp: 'PaoloBonzini 10/3/2009 02:07'!
initialize
	self connectionPoolSize: self class defaultConnectionPoolSize.
	self initSites.
	self initServers.! !

Object subclass: #SwazooStream
	instanceVariableNames: 'socket readBuffer readPtr readEnd writeBuffer writePtr writeEnd chunked'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

!SwazooStream class methodsFor: 'instance creation' stamp: ''!
connectedPair
	^SwazooSocket connectedPair collect: [:each | self socket: each]
! !

!SwazooStream class methodsFor: 'defaults' stamp: ''!
defaultBufferSize
        ^8000
! !

!SwazooStream class methodsFor: 'instance creation' stamp: ''!
on: aString
	"use only for testing!!!!"
	^self new setInputString: aString
! !

!SwazooStream class methodsFor: 'defaults' stamp: ''!
preambleSize
	^6
! !

!SwazooStream class methodsFor: 'instance creation' stamp: ''!
socket: aSwazooSocket
	^self new setSocket: aSwazooSocket
! !

!SwazooStream methodsFor: 'accessing-reading' stamp: ''!
anyDataReady
	"wait for data and return true if any data ready. On VW somethimes happen that data
	 receipt is signaled but no data is actually received"
	self fillBuffer.
	^readPtr <= readEnd
! !

!SwazooStream methodsFor: 'accessing-reading' stamp: ''!
atEnd
	"TCP socket data never ends!!!!"
	^false
! !

!SwazooStream methodsFor: 'initialize-release' stamp: ''!
close
	"close TCP socket and relase buffers"
	self socket close.
	self nilWriteBuffer.
	self nilReadBuffer	"to GC buffers"
! !

!SwazooStream methodsFor: 'chunked encoding' stamp: ''!
closeChunk
	"a zero sized chunk determine and end of chunked data and also response"

        | written |
        "first crlf ends 0 length line, second crlf ends whole response"
        written := self socket
                    writeFrom: #(48 13 10 13 10) asByteArray
                    startingAt: 1
                    for: 5.
        written = 5 ifFalse: [self error: 'socket write error']
! !

!SwazooStream methodsFor: 'initialize-release' stamp: ''!
closeResponse
	"for chunked response: close it by sending null chunk"

	"do a bit cleanup after response is sent"
	self flush.
	self isChunked ifTrue: [self closeChunk; resetChunked].
! !

!SwazooStream methodsFor: 'mime boundary' stamp: 'PaoloBonzini 10/4/2009 11:14'!
copyBufferTo: anIndex
	"from current position to desired index"
	| start |
	start := readPtr.
	readPtr := anIndex.
	^(ByteArray new: anIndex - start)
		replaceFrom: 1 to: anIndex - start
		with: readBuffer startingAt: start
! !

!SwazooStream methodsFor: 'accessing-writing' stamp: ''!
cr
	self nextPutByte: 13
! !

!SwazooStream methodsFor: 'accessing-writing' stamp: ''!
crlf
	self
	    cr;
	    lf
! !

!SwazooStream methodsFor: 'buffer size' stamp: ''!
enlargeReadBuffer: anInteger
	anInteger < readBuffer size ifTrue: [ ^self ].
	readBuffer := (readBuffer class new: anInteger)
	    replaceFrom: 1
	        to: readBuffer size
	        with: readBuffer
	        startingAt: 1;
	    yourself
! !

!SwazooStream methodsFor: 'private' stamp: ''!
fillBuffer
	readPtr > readEnd ifFalse: [^self].
	self socket isNil ifTrue: [^self].	"if SwazooStream is used for tests only"
	readPtr := 1.
        readEnd := self socket
                    readInto: readBuffer
                    startingAt: 1
                    for: readBuffer size.       "nr. of actuall bytes read"
        readEnd = 0
            ifTrue:
                [SwazooStreamNoDataError
                    raiseSignal: 'No data available.  Socket probably closed']
! !

!SwazooStream methodsFor: 'chunked encoding' stamp: ''!
fillPreamble
	| size length start |
	size := self writeDataSize.
	"preamble has no room for bigger chunk..."
        size > 65535 ifTrue: [self error: 'chunk too long!!'].
        length := size printStringRadix: 16.

        SpEnvironment isSqueak ifTrue: [length := length copyFrom: 4].     "trim 16r"
	writeBuffer
	    replaceFrom: 1 to: length size
	    with: length
	    startingAt: 1.
	writeBuffer
	    replaceFrom: length size + 1 to: 6
	    with: #[32 32 32 32 13 10]
	    startingAt: length size + 1.
	writePtr := 1
! !

!SwazooStream methodsFor: 'initialize-release' stamp: ''!
flush
	"actually write to the tcp socket and clear write buffer"
	| written |
	self socket isNil ifTrue: [^nil].	"for simulations and tests"

	self isChunked ifTrue: [self fillPreamble].
	[writeEnd < writePtr] whileFalse:
	    [written := self socket
                            writeFrom: writeBuffer
                            startingAt: writePtr
                            for: writeEnd - writePtr + 1.
            writePtr := writePtr + written].
	writePtr := self class preambleSize + 1.
	writeEnd := self class preambleSize.
! !

!SwazooStream methodsFor: 'mime boundary' stamp: ''!
indexOfBoundary: aBoundaryBytes
	"index of boundary start, beeing full boundary or part at the end of buffer. 0 if not found"
	| inx innerInx firstInx |
	inx := readPtr.
	[inx <= readEnd] whileTrue: 
		[innerInx := 1.
		firstInx := inx.
		[(aBoundaryBytes at: innerInx) = (readBuffer at: inx)] whileTrue: 
			[innerInx = aBoundaryBytes size ifTrue: [^firstInx].	"full boundary found"
			inx = readEnd ifTrue: [^firstInx].	"partial boundary at the edge of buffer found"
			inx := inx + 1.
			innerInx := innerInx + 1].
		inx := inx + 1].
	^0
! !

!SwazooStream methodsFor: 'chunking' stamp: ''!
isChunked
	"sending in chunks (transfer encoding: chunked)"
	^chunked notNil and: [chunked]
! !

!SwazooStream methodsFor: 'chunking' stamp: ''!
isFull
	"sending in chunks (transfer encoding: chunked)"
	^writeEnd >= writeBuffer size
! !

!SwazooStream methodsFor: 'accessing-writing' stamp: ''!
lf
	self nextPutByte: 10
! !

!SwazooStream methodsFor: 'accessing-reading' stamp: 'PaoloBonzini 10/4/2009 11:52'!
next
	self fillBuffer.
	^(readBuffer at: (readPtr := readPtr + 1) - 1) asCharacter
! !

!SwazooStream methodsFor: 'accessing-reading' stamp: ''!
next: anInteger
	| array at n |
	array := String new: anInteger.
	at := 1.
	[ at <= anInteger ] whileTrue: [
	    self fillBuffer.
	    n := readEnd - readPtr + 1 min: anInteger - at + 1.
	    array replaceFrom: at to: at + n - 1 with: readBuffer startingAt: readPtr.
	    readPtr := readPtr + n.
	    at := at + n].
	^array
! !

!SwazooStream methodsFor: 'private-stream' stamp: 'PaoloBonzini 10/4/2009 11:46'!
nextByte
	self fillBuffer.
	^readBuffer at: (readPtr := readPtr + 1) - 1
! !

!SwazooStream methodsFor: 'private-stream' stamp: ''!
nextBytes: anInteger
	| array at n |
	array := ByteArray new: anInteger.
	at := 1.
	[ at <= anInteger ] whileTrue: [
	    self fillBuffer.
	    n := readEnd - readPtr + 1 min: anInteger - at + 1.
	    array replaceFrom: at to: at + n - 1 with: readBuffer startingAt: readPtr.
	    readPtr := readPtr + n.
	    at := at + n].
	^array
! !

!SwazooStream methodsFor: 'accessing-reading' stamp: 'PaoloBonzini 10/4/2009 11:56'!
nextLine
	| stream |
	stream := WriteStream on: (ByteArray new: 50).
	self writeNextLineTo: stream.
	^stream contents asString
! !

!SwazooStream methodsFor: 'accessing-writing' stamp: 'PaoloBonzini 10/4/2009 11:47'!
nextPut: aCharacterOrInteger
	self isFull ifTrue: [self flush].
	writeBuffer at: (writeEnd := writeEnd + 1) put: aCharacterOrInteger asInteger.
	^aCharacterOrInteger
! !

!SwazooStream methodsFor: 'accessing-writing' stamp: ''!
nextPutAll: aByteStringOrArray
	| at n |
	at := 1.
	[ at <= aByteStringOrArray size ] whileTrue: [
	    self isFull ifTrue: [self flush].
	    n := writeBuffer size - writeEnd min: aByteStringOrArray size - at + 1.
	    writeBuffer replaceFrom: writeEnd + 1 to: writeEnd + n with: aByteStringOrArray startingAt: at.
	    writeEnd := writeEnd + n.
	    at := at + n].
	^aByteStringOrArray
! !

!SwazooStream methodsFor: 'accessing-reading' stamp: ''!
nextPutAllBufferOn: aStream
	aStream next: readEnd - readPtr + 1 putAll: readBuffer startingAt: readPtr.
	readPtr := readEnd + 1
! !

!SwazooStream methodsFor: 'private-stream' stamp: 'PaoloBonzini 10/4/2009 11:47'!
nextPutByte: aByte
	self isFull ifTrue: [self flush].
	^writeBuffer at: (writeEnd := writeEnd + 1) put: aByte
! !

!SwazooStream methodsFor: 'private-stream' stamp: ''!
nextPutBytes: aByteArray
	^self nextPutAll: aByteArray
! !

!SwazooStream methodsFor: 'accessing-writing' stamp: ''!
nextPutLine: aByteStringOrArray
	self nextPutAll: aByteStringOrArray.
	self crlf
! !

!SwazooStream methodsFor: 'accessing-reading' stamp: 'PaoloBonzini 10/4/2009 11:53'!
nextUnfoldedLine
	| stream ch |
	stream := WriteStream on: (ByteArray new: 50).
	self writeNextLineTo: stream.
	stream position = 0
	    ifFalse: 
		[
		[ch := self peek.
		ch notNil and: [ch == Character space or: [ch == Character tab]]] 
			whileTrue: [self writeNextLineTo: stream]].
	^stream contents asString
! !

!SwazooStream methodsFor: 'initialize-release' stamp: ''!
nilReadBuffer
	"to release memory"
	readBuffer := nil
! !

!SwazooStream methodsFor: 'initialize-release' stamp: ''!
nilWriteBuffer
	"to release memory"
	writeBuffer := nil
! !

!SwazooStream methodsFor: 'accessing-reading' stamp: 'PaoloBonzini 10/4/2009 11:52'!
peek
	self anyDataReady ifFalse: [^nil].
	^(readBuffer at: readPtr) asCharacter
! !

!SwazooStream methodsFor: 'private-stream' stamp: 'PaoloBonzini 10/4/2009 11:46'!
peekByte
	self anyDataReady ifFalse: [^nil].
	^readBuffer at: readPtr
! !

!SwazooStream methodsFor: 'private' stamp: ''!
print: anObject
	anObject printOn: self
! !

!SwazooStream methodsFor: 'accessing' stamp: ''!
readBuffer
	^readBuffer
! !

!SwazooStream methodsFor: 'private' stamp: ''!
readBuffer: aByteArray ready: dataLength
	readBuffer := aByteArray.
	readPtr := 1.
	readEnd := dataLength.
! !

!SwazooStream methodsFor: 'chunking' stamp: ''!
resetChunked
	"sending in chunks (transfer encoding: chunked)"
	chunked := false
! !

!SwazooStream methodsFor: 'chunking' stamp: ''!
setChunked
	"sending in chunks (transfer encoding: chunked)"
	chunked := true
! !

!SwazooStream methodsFor: 'private' stamp: 'PaoloBonzini 10/4/2009 11:46'!
setInputString: aCollection
	self readBuffer: aCollection asByteArray ready: aCollection size.
	self writeBuffer: (ByteArray new: self class defaultBufferSize).
! !

!SwazooStream methodsFor: 'private' stamp: 'PaoloBonzini 10/4/2009 11:46'!
setSocket: aSwazooSocket
	self socket: aSwazooSocket.
	self readBuffer: (ByteArray new: self class defaultBufferSize) ready: 0.
	self writeBuffer: (ByteArray new: self class defaultBufferSize).
! !

!SwazooStream methodsFor: 'mime boundary' stamp: 'PaoloBonzini 10/4/2009 11:48'!
signsOfBoundary: aBoundaryBytes
	"detect as fast as possible if any if not all MIME part boundary is present in buffer contents"

	"return number of bundary bytes detected, 0 = no boundary"
	| first index |
	first := aBoundaryBytes first asInteger.
	"fast test"
	((readPtr + 1 to: readEnd) 
	    contains: [:inx | (readBuffer at: inx) = first]) ifFalse: [^0].
	"full or partial boundary on the edge of buffer test"
	index := self indexOfBoundary: aBoundaryBytes.	"index of full, or partial boundary at the edge"
	index = 0 ifTrue: [^0].	"no boundary found"
	readEnd - index >= aBoundaryBytes size ifTrue: [^aBoundaryBytes size].	"full boundary detected"
	^readEnd - index + 1	"partial boundary at the end of buffer"
! !

!SwazooStream methodsFor: 'accessing-reading' stamp: ''!
skip: anInteger
	| n skipped |
	n := anInteger.
	[
	    skipped := n min: (readEnd - readPtr + 1).
	    readPtr := readPtr + skipped.
	    n := n - skipped.
	    n > 0
	] whileTrue: [self fillBuffer]
! !

!SwazooStream methodsFor: 'private' stamp: ''!
socket
	^socket
! !

!SwazooStream methodsFor: 'private' stamp: ''!
socket: aSocket
	socket := aSocket
! !

!SwazooStream methodsFor: 'accessing-writing' stamp: ''!
space
	self nextPutByte: 32
! !

!SwazooStream methodsFor: 'mime boundary' stamp: ''!
startsWith: aPartialBoundaryBytes
	"is remaining part of MIME part boundary at the start of buffer?"
	1 to: aPartialBoundaryBytes size
	    do: 
		[:inx | 
		(readBuffer at: readPtr + inx) = (aPartialBoundaryBytes at: inx) 
		    ifFalse: [^false]].
	^true
! !

!SwazooStream methodsFor: 'accessing-reading' stamp: 'PaoloBonzini 10/4/2009 11:47'!
upTo: aCharacterOrByte
	| targetChar result r ws |
	targetChar := aCharacterOrByte asInteger. "###"
	r := readBuffer indexOf: targetChar startingAt: readPtr ifAbsent: [0].
	r = 0 ifFalse: [result := self next: r - readPtr. self next. ^result].

	ws := String new writeStream.
	[self nextPutAllBufferOn: ws.
	self fillBuffer.
	r := readBuffer indexOf: targetChar startingAt: readPtr ifAbsent: [0].
	r = 0] whileTrue.

	ws next: r putAll: readBuffer startingAt: 1.
	readPtr := r + 1.
	^ws contents
! !

!SwazooStream methodsFor: 'private' stamp: ''!
writeBuffer: aByteArray
	writeBuffer := aByteArray.
	writePtr := self class preambleSize + 1.
	writeEnd := self class preambleSize.
! !

!SwazooStream methodsFor: 'accessing-writing' stamp: ''!
writeBufferContents
	^writeBuffer copyFrom: writePtr to: writeEnd
! !

!SwazooStream methodsFor: 'chunked encoding' stamp: ''!
writeDataSize
	^writeEnd - writePtr + 1
! !

!SwazooStream methodsFor: 'accessing-reading' stamp: 'PaoloBonzini 10/4/2009 11:55'!
writeNextLineTo: aStream
	| r |
	[r := readBuffer indexOf: 13 startingAt: readPtr ifAbsent: [0].
	r = 0] whileTrue: [
	    self nextPutAllBufferOn: aStream.
	    self fillBuffer].

	aStream next: r - readPtr putAll: readBuffer startingAt: readPtr.
	readPtr := r + 1.
	self peekByte = 10	"skip remaining linefeed"
	    ifTrue: [readPtr := readPtr + 1]
	    ifFalse: [SwazooHTTPParseError raiseSignal: 'CR without LF']
! !

!SwazooServerTest methodsFor: 'testing' stamp: 'PaoloBonzini 10/4/2009 11:18'!
testStartingOnTwoPorts
	"and all ip interfaces, any host"
	
	| server nrServers site1 site2 |
	server _ SwazooServer singleton.
	nrServers _ server servers size.
	[site1 _ server startOn: 4924.
	site2 _ server startOn: 4925.
	self assert: site1 isServing.
	self assert: site2 isServing.
	self assert: server servers size = (nrServers + 2).
	server stopOn: 4924.
	server stopOn: 4925.
	self assert: site1 isServing not.
	self assert: site2 isServing not.
	self assert: server servers size = nrServers]
		ensure:
			[site1 stop.
			site2 stop.
			server
				removeSite: site1;
				removeSite: site2]! !

!HTTPRequestLine methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
httpVersion: anArray 
	httpVersion := anArray.
	^self! !

!HTTPRequestLine methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
method: aString
"For development testing only"
	method := aString.
	^self! !

!HTTPRequestLine methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
requestURI: aString 
	"Development testing only!!"

	requestURI := aString.
	^self! !

!FileResponse methodsFor: 'private-printing' stamp: 'PaoloBonzini 10/2/2009 18:47'!
contentSize
	^self entity notNil ifTrue: [self entity value fileSize] ifFalse: [0]! !

!FileResponse methodsFor: 'accessing' stamp: 'PaoloBonzini 10/2/2009 18:47'!
entity: aMimeObject 
	entity := aMimeObject.
	self contentType: self entity contentType.! !

!HTTPResponseTest methodsFor: 'testing' stamp: 'PaoloBonzini 10/4/2009 10:29'!
testInternalServerError
	| ws rs |
	response := HTTPResponse internalServerError.
	ws := SwazooStream on: String new.
	HTTPPrinter new response: response; stream: ws; printStatus.
	rs := SwazooStream on: ws writeBufferContents.
	self assert: rs nextLine = 'HTTP/1.1 500 Internal Server Error'! !

!HTTPResponseTest methodsFor: 'testing' stamp: 'PaoloBonzini 10/4/2009 10:30'!
testOK
	| ws rs |
	response := HTTPResponse ok.
	ws := SwazooStream on: String new.
	HTTPPrinter new response: response; stream: ws; printStatus.
	rs := SwazooStream on: ws writeBufferContents.
	self assert: rs nextLine = 'HTTP/1.1 200 OK'! !

!HTTPRequest class methodsFor: 'instance creation' stamp: 'PaoloBonzini 10/2/2009 18:41'!
classFor: aString
	aString = 'GET' ifTrue: [^HTTPGet       "most used anyway"].
	aString = 'POST' ifTrue: [^HTTPPost     "second most used"].

	^self subclasses detect: [:each | each methodName = aString]
		ifNone: [^HTTPException notImplemented].
! !

!HTTPRequest class methodsFor: 'instance creation' stamp: 'PaoloBonzini 10/2/2009 18:42'!
readFrom: aSwazooStream 
	^HTTPReader readFrom: aSwazooStream ! !

!HTTPRequest methodsFor: 'testing' stamp: 'PaoloBonzini 10/2/2009 22:03'!
isKeepAlive
	| header result |
	result := self isHttp10 not.
	header := self connection.
	header isNil ifFalse: [
		('*Keep-Alive*' match: header) ifTrue: [result := true].
		('*Close*' match: header) ifTrue: [result := false]].
	^result! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: 'PaoloBonzini 10/2/2009 22:03'!
keepAlive
	"how many seconds a connection must be kept alive"
	^(self headers fieldNamed: 'Keep-Alive' ifNone: [^nil]) value! !

!HTTPRequest methodsFor: 'parsing' stamp: 'PaoloBonzini 10/2/2009 18:43'!
readFrom: aSwazooStream
	"^self
	
	Parses the entity."! !

!HTTPRequest methodsFor: 'initialize-release' stamp: 'PaoloBonzini 10/2/2009 18:42'!
requestLine: aRequestLine
	requestLine := aRequestLine.
! !

!HTTPPostTest methodsFor: 'requests' stamp: 'PaoloBonzini 10/4/2009 10:23'!
fileContents
	"HTTPRequestTest new fileContents"
	| stream |
	stream := SwazooStream on: String new.
	stream
		nextPutLine: 'BEGIN:VCALENDAR';
		nextPutLine: 'PRODID:-//Squeak-iCalendar//-';
		nextPutLine: 'VERSION:2.0';
		nextPutLine: 'X-WR-CALNAME:test';
		nextPutLine: 'METHOD:PUBLISH';
		nextPutLine: 'BEGIN:VEVENT';
		nextPutLine: 'UID:an event with a start date and not date and time';
		nextPutLine: 'CATEGORIES:category1,category2';
		nextPutLine: 'CREATED:20050501T110231Z';
		nextPutLine: 'SEQUENCE:0';
		nextPutLine: 'SUMMARY:aTitle';
		nextPutLine: 'PRIORITY:5';
		nextPutLine: 'DTSTART;VALUE=DATE:20050425';
		nextPutLine: 'END:VEVENT';
		nextPutLine: 'END:VCALENDAR'.

	^stream writeBufferContents asString! !

!HTTPPostTest methodsFor: 'requests' stamp: 'PaoloBonzini 10/4/2009 10:26'!
postDashes
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST  /document/aab.html HTTP/1.1';
		nextPutLine: 'Host: biart.eranova.si';
		nextPutLine: 'Content-Type: multipart/form-data; boundary= boundary';
		nextPutLine: 'Content-Length: 149';
            crlf;
		nextPutLine: '--boundary';
		nextPutLine: 'Content-Disposition: form-data; name="id5273"';
		crlf; 
		nextPutLine: '----';
		nextPutLine: '--boundary';
		nextPutLine: 'Content-Disposition: form-data; name="field2"';
		crlf; 
		nextPutLine: '- --';
		nextPutLine: '--boundary--'.

	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents)! !

!HTTPPostTest methodsFor: 'requests' stamp: 'PaoloBonzini 10/4/2009 10:27'!
postEmpty
	"post entity with empty value"
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST  /document/aab.html HTTP/1.1';
		nextPutLine: 'Host: biart.eranova.si';
		nextPutLine: 'Content-Type: multipart/form-data; boundary= boundary';
		nextPutLine: 'Content-Length: 75';
            crlf;
		nextPutLine: '--boundary';
		nextPutLine: 'Content-Disposition: form-data; name="id5273"';
		crlf; 
		nextPutLine: '';
		nextPutLine: '--boundary--'.

	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents)! !

!HTTPPostTest methodsFor: 'requests' stamp: 'PaoloBonzini 10/4/2009 10:27'!
postFile
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST  /document/aab.html HTTP/1.1';
		nextPutLine: 'Connection: Keep-Alive';
		nextPutLine: 'User-Agent: Mozilla/4.72 [en] (X11; I; Linux 2.3.51 i686)';
		nextPutLine: 'Host: biart.eranova.si';
		nextPutLine: 'Referer: http://www.bar.com/takeMeThere.html';
		nextPutLine: 'Content-Type: multipart/form-data; boundary= -----------------20752836116568320241700153999';
		nextPutLine: 'Content-Length: ', (527+self fileContents size) printString;
            crlf;
		nextPutLine: '-------------------20752836116568320241700153999';
		nextPutLine: 'Content-Disposition: form-data; name="id5273"';
		crlf; 
		nextPutLine: 'main';
		nextPutLine: '-------------------20752836116568320241700153999';
		nextPutLine: 'Content-Disposition: form-data; name="field2"';
		crlf; crlf;
		nextPutLine: '-------------------20752836116568320241700153999';
		nextPutLine: 'Content-Disposition: form-data; name="field7"; filename="event.ical"';
		nextPutLine: 'Content-Type: application/octet-stream';
		crlf;

		nextPutAll: self fileContents;

		crlf;
		nextPutLine: '-------------------20752836116568320241700153999';
		nextPutLine: 'Content-Disposition: form-data; name="attach"';
		crlf;
		nextPutLine: 'Attach';
		nextPutLine: '-------------------20752836116568320241700153999--'.

	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents)! !

!HTTPPostTest methodsFor: 'requests' stamp: 'PaoloBonzini 10/4/2009 10:27'!
postPreambleEpilogue
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST  /document/aab.html HTTP/1.1';
		nextPutLine: 'Host: biart.eranova.si';
		nextPutLine: 'Content-Type: multipart/form-data; boundary= boundary';
		nextPutLine: 'Content-Length: 146';
            crlf;
		nextPutLine: 'This is a multi-part message in MIME format';
		nextPutLine: '--boundary';
		nextPutLine: 'Content-Disposition: form-data; name="id5273"';
		crlf; 
		nextPutLine: 'main';
		nextPutLine: '--boundary--';
		nextPutLine: 'This is the epilogue'.

	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents)! !

!HTTPPostTest methodsFor: 'requests' stamp: 'PaoloBonzini 10/4/2009 10:27'!
postSimple
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST  /document/aab.html HTTP/1.1';
		nextPutLine: 'Host: biart.eranova.si';
		nextPutLine: 'Content-Type: multipart/form-data; boundary= boundary';
		nextPutLine: 'Content-Length: 79';
            crlf;
		nextPutLine: '--boundary';
		nextPutLine: 'Content-Disposition: form-data; name="id5273"';
		crlf; 
		nextPutLine: 'main';
		nextPutLine: '--boundary--'.

	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents)! !

!HTTPPostTest methodsFor: 'requests' stamp: 'PaoloBonzini 10/4/2009 10:27'!
postUrlEncoded
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST  /document/aab.html HTTP/1.1';
		nextPutLine: 'Host: biart.eranova.si';
		nextPutLine: 'Content-Type: application/x-www-form-urlencoded';
		nextPutLine: 'Content-Length: 36';
            crlf;
		nextPutAll: 'home=Cosby+one&favorite+flavor=flies'.

	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents)! !

!HTTPPostTest methodsFor: 'testing-mime parsing' stamp: 'PaoloBonzini 10/4/2009 11:16'!
testBlockCopy
	"streaming with 8k blocks for performance"
	"this is just a basic test with content shorter that one block"
	| boundary message in out |
	boundary := '--boundary--'.
	message := 'just something'. 
	in := SwazooStream on: message, self crlf, boundary.
	out := WriteStream on: ByteArray new.
	HTTPPost new blockStreamingFrom: in to: out until: boundary.
	self assert: out contents asString = message.! !

!HTTPPostTest methodsFor: 'testing-posts' stamp: 'PaoloBonzini 10/4/2009 10:27'!
testPostRawEntity
	| requestStream post |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST /foobar HTTP/1.0';
		nextPutLine: 'Host: foo.com';
		nextPutLine: 'Content-Type: text/plain';
		nextPutLine: 'Content-Length: 12';
		crlf;
		nextPutLine: 'Hello, World'.
	post := HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents).
	self assert: post isPostDataEmpty.
	self assert: post entityBody = 'Hello, World'! !

!HTTPPostTest methodsFor: 'testing-posts' stamp: 'PaoloBonzini 10/4/2009 10:31'!
testPostUrlEncodedData
	| requestStream post |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST / HTTP/1.1';
		nextPutLine: 'Host: foo.com';
		nextPutLine: 'Content-Type: application/x-www-form-urlencoded';
		nextPutLine: 'Content-Length: 31';
		crlf;
		nextPutLine: 'address=+fs&product=&quantity=1'.
	post := HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents).
	self assert: (post postDataAt: 'address') value = ' fs'.
	self assert: (post postDataAt: 'product') value = ''.
	self assert: (post postDataAt: 'quantity') value = '1'! !

!HTTPRequestTest methodsFor: 'requests-gets' stamp: 'PaoloBonzini 10/4/2009 10:27'!
basicGet
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET / HTTP/1.1';
		nextPutLine: 'Host: foo.com';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents)! !

!HTTPRequestTest methodsFor: 'requests-gets' stamp: 'PaoloBonzini 10/4/2009 10:28'!
basicGetHTTP10
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET / HTTP/1.0';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents)! !

!HTTPRequestTest methodsFor: 'requests-gets' stamp: 'PaoloBonzini 10/4/2009 10:28'!
basicGetHTTP10Keepalive
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET / HTTP/1.0';
		nextPutLine: 'Connection: Keep-Alive';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents)! !

!HTTPRequestTest methodsFor: 'requests-gets' stamp: 'PaoloBonzini 10/4/2009 10:28'!
basicHead
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'HEAD / HTTP/1.1';
		nextPutLine: 'Host: foo.com';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents)! !

!HTTPRequestTest methodsFor: 'requests-gets' stamp: 'PaoloBonzini 10/4/2009 10:29'!
fullGet
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET /aaa/bbb/ccc.html?foo=bar&baz=quux HTTP/1.1';
		nextPutLine: 'Connection: Keep-Alive';
		nextPutLine: 'User-Agent: Mozilla/4.72 [en] (X11; I; Linux 2.3.51 i686)';
		nextPutLine: 'Host: foo.com:8888';
		nextPutLine: 'Referer: http://www.bar.com/takeMeThere.html';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents)! !

!HTTPRequestTest methodsFor: 'requests-gets' stamp: 'PaoloBonzini 10/4/2009 10:29'!
getMultiValueHeader
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET /aaa/bbb/ccc.html?foo=bar&baz=quux HTTP/1.1';
		nextPutLine: 'Content-Type: multipart/form-data; boundary= --boundary';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents)! !

!HTTPRequestTest methodsFor: 'requests-gets' stamp: 'PaoloBonzini 10/4/2009 10:29'!
portedGet
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET / HTTP/1.1';
		nextPutLine: 'Host: foo.com:8888';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents)! !

!HTTPRequestTest methodsFor: 'testing-other' stamp: 'PaoloBonzini 10/4/2009 10:29'!
testMissingContentType
	| requestStream result |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST /foobar HTTP/1.0';
		nextPutLine: 'Host: foo.com';
"	      nextPutLine: 'Content-Type: text/plain'. <-- this is missing!! - and should be for this test"
		nextPutLine: 'Content-Length: 12';
		crlf;
		nextPutLine: 'Hello, World'.
	result := SpExceptionContext 
		for: [(HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents)) 
				ensureFullRead] "because of defered post data parsing"
		on: SpError
		do: [:ex | ex].
	self assert: result class == SwazooHTTPPostError.
	^self! !

!HTTPRequestTest methodsFor: 'testing-other' stamp: 'PaoloBonzini 10/4/2009 10:30'!
testNoEqualsQueries
	"The last assert here used to check that 'request queryAt: 'WSDL'' is nil, but a test for an empty string is more consistent with query argument formats."

	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET /test/typed.asmx?WSDL HTTP/1.1';
		nextPutLine: 'Host: foo.com:8888';
		crlf.
	request := HTTPRequest readFrom: (SwazooStream on: requestStream writeBufferContents).
	self assert: (request includesQuery: 'WSDL').
	self assert: (request queryAt: 'WSDL') isEmpty! !

!SwazooBenchmarks methodsFor: 'initialize-release' stamp: 'PaoloBonzini 10/4/2009 10:30'!
initContent
	| response ws |
	response := HTTPResponse ok.
	response entity: (ByteArray new: self contentSize withAll: 16r55).
	ws := SwazooStream on: String new.
	HTTPPrinter writeResponse: response to: ws.
	content := ws writeBufferContents.! !

!TestPseudoSocket methodsFor: 'stream-toServer' stamp: 'PaoloBonzini 10/4/2009 10:41'!
upTo: aCharacter 
	"a ByteString
For some reason, we have to look for a character in a ByteStream - this is a Swazoo thing."

	^self byteStreamToServer upTo: aCharacter asInteger! !
