
!HTTPPost methodsFor: 'private-parsing support' stamp: 'janko 8/4/2008 13:16'!
blockStreamingFrom: aSwazooStream to: outStream until: aBoundaryBytes "detected"
	"copy by 8k blocks for optimal performance until a boundary of MIME part is detected"
	"Finish appropriatelly streaming at the end (skip crlf etc.)"
	| boundary start nrOfBoundary contents inPrevious remainingBoundary boundaryIndex |
	boundary := aSwazooStream readBuffer isBinary 
		ifTrue: [aBoundaryBytes asByteArray] ifFalse: [aBoundaryBytes asByteString].
	start := self readPosition.
	[true] whileTrue:
		[nrOfBoundary := 0.
		[nrOfBoundary = 0] whileTrue:
			[nrOfBoundary := aSwazooStream readBuffer signsOfBoundary: boundary.
			nrOfBoundary = 0 ifTrue: "no boundary in current buffer content"
				[contents := aSwazooStream readBuffer readContents.
				outStream nextPutAll: contents. 
				self incReadPosition: contents size.
				self checkToEnlargeBufferIn: aSwazooStream from: start. "for effective streaming"
				aSwazooStream fillBuffer] ].
		"copy and stream out content up to potential boundary"
		boundaryIndex := aSwazooStream readBuffer indexOfBoundary: boundary.
		inPrevious := aSwazooStream readBuffer copyBufferTo: boundaryIndex-1.
		outStream nextPutAll: 
			(inPrevious copyFrom: 1 to: (inPrevious size-2 max: 0) "without potential crlf"). 
		self incReadPosition: inPrevious size. "potential crlf included!!"

		nrOfBoundary = boundary size ifTrue: "full boundary detected, lets finish here"
			[aSwazooStream skip: boundary size. "skip boundary"
			self incReadPosition: boundary size.
			^true].  "streaming complete"

		self incReadPosition: nrOfBoundary.
		aSwazooStream fillBuffer. "let's get next buffer"
		remainingBoundary := boundary copyFrom: nrOfBoundary+1 to: boundary size.
		(aSwazooStream readBuffer startsWith:  remainingBoundary) ifTrue: "bound. ends in next buff?"
			[aSwazooStream skip: remainingBoundary size + 2.  "skip remaining bound. and crlf"
			self incReadPosition: remainingBoundary size + 2.
			^true]. "streaming complete"

		outStream 
			nextPutAll: (inPrevious copyFrom: inPrevious size-2 to: inPrevious size);  "potential crlf"
			nextPutAll: (boundary copyFrom: 1 to: nrOfBoundary).  "boundary part in prev.buff."
		] "continue from the start"! !

!HTTPPost methodsFor: 'private-parsing support' stamp: 'janko 8/3/2008 17:21'!
checkToEnlargeBufferIn: aSwazooStream from: startPosition
	"enlarge buffer to 1MB (if not already) if more than 100KB already read"
	aSwazooStream readBuffer isEnlarged ifTrue: [^nil].
	(self readPosition - startPosition) > 100000 "about 100KB" 
		ifTrue: [aSwazooStream readBuffer resizeEnlarge].! !

!HTTPPost methodsFor: 'parsing' stamp: 'mivsek 7/19/2007 22:12'!
for: aRequestLine readFrom: aSwazooStream 
	super for: aRequestLine readFrom: aSwazooStream.
	self initPostDataFor: aSwazooStream.
"	self parsePostDataFrom: aSwazooStream." "defered until first access of postData!! "! !

!HTTPPost methodsFor: 'private-parsing' stamp: 'jm 2/8/2009 14:55'!
multipartDataFrom: aSwazooStream
	"read all mime parts and put them in postData" 
	"read directly from stream, without intermediate buffers"
	| contentTypeField boundary part |
	contentTypeField := self headers fieldOfClass: ContentTypeField
				ifNone: [^aSwazooStream nextBytes: self contentLength]. "just skip"
	boundary := contentTypeField transferCodings at: 'boundary'
				ifAbsent: [^aSwazooStream nextBytes: self contentLength]. "just skip"
	self skipMimePreambleAndBoundary: boundary from: aSwazooStream. 
	part := #something. 
	[part notNil] whileTrue: 
		[part := self partFromStream: aSwazooStream boundary: boundary.
		part notNil ifTrue: [self postDataAt: part key put: part value]].
	self skipMimeEpilogueFrom: aSwazooStream. "all to the end  as defined by contentLegth"
	aSwazooStream readBuffer isEnlarged  "if MIME part larger that 100KB was read"
		ifTrue: [aSwazooStream readBuffer resizeShrink]. "that is, release memory"! !

!HTTPPost methodsFor: 'private-parsing' stamp: 'janko 7/26/2008 18:48'!
readEntityFrom: aSwazooStream datum: aDatum boundary: aBoundaryBytes
	"read one entity from a stream and put into datum. Stream it if streamed. Also call a block 
	(if any) just before start of streaming, with a datum as parameter. This block can then set 
	a write stream in datum (for instance open a output file and stream on it)"
	| outStream |
	aDatum writeBlock notNil 
		ifTrue: [aDatum writeBlock value: aDatum]. "this should set writeStream if not already!!" 
	outStream := (aDatum isStreamed and: [aDatum writeStream notNil])
		ifTrue: [aDatum writeStream] ifFalse: [WriteStream on: ByteArray new].
	self blockStreamingFrom: aSwazooStream to: outStream until: ('--', aBoundaryBytes). "efficient streaming"
	aDatum isStreamed not ifTrue: "otherwise entity is already streamed to the output"
		[aDatum value: outStream contents asString].
	^aDatum! !

!HeaderField class methodsFor: 'instance creation' stamp: ' 14/4/07 23:10'!
fromLine: aString 
	| sourceStream fieldName fieldValue fieldClass |
	sourceStream := ReadStream on: aString.
	fieldName := (HTTPString trimBlanksFrom: (sourceStream upTo: $:)) 
				asUppercase.
	fieldClass := self classForFieldName: fieldName.
	fieldValue := HTTPString trimBlanksFrom: sourceStream upToEnd.
	^fieldClass newForFieldName: fieldName withValueFrom: fieldValue! !

!HTTPResponse class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
statusTextForCode: aNumber
	^StatusCodes at: aNumber ifAbsent: ['']! !

!HTTPResponse methodsFor: 'accessing-headers' stamp: 'mivsek 12/31/2007 15:50'!
contentLength: anInteger
	self headers addField: (HTTPContentLengthField new contentLength: anInteger).
	^self! !

!HTTPResponse methodsFor: 'private-sending' stamp: ' 14/4/07 23:10'!
crlfOn: aStream 
	aStream
		nextPut: Character cr;
		nextPut: Character lf! !

!HTTPResponse methodsFor: 'private-sending' stamp: ' 14/4/07 23:10'!
endHeaderOn: aStream 
	self crlfOn: aStream! !

!HTTPResponse methodsFor: 'private-sending' stamp: ' 14/8/07 21:49'!
printChunkedTransferEncodingOn: aStream 
	aStream nextPutAll: 'Transfer-Encoding: chunked'.
	self crlfOn: aStream! !

!HTTPResponse methodsFor: 'private-sending' stamp: 'mivsek 12/31/2007 15:52'!
printContentLengthOn: aSwazooStream 
	"it is also added to headers. It is added so late because to be printed last, 
	just before body starts"
	self contentLength: self contentSize.
	(self headers fieldNamed: 'Content-length') printOn: aSwazooStream.
	self crlfOn: aSwazooStream! !

!HTTPResponse methodsFor: 'private-sending' stamp: 'mivsek 7/22/2007 19:18'!
printHeadersOn: aSwazooStream 
	"^self
Write the headers (key-value pairs) to aStream.  The key
must be a String."

	self headers fields do: 
			[:aField | 
			aField printOn: aSwazooStream.
			self crlfOn: aSwazooStream]! !

!HTTPResponse methodsFor: 'private-sending' stamp: 'mivsek 12/31/2007 15:53'!
printStatusOn: aSwazooStream 
	| version |
	StatusCodes at: self code ifAbsent: [self class initialize]. "if some new status codes was added           													with #postInitialize method later"
	version := (self task isNil or: [self task request isNil or: [self task request isHttp11]]) 
		ifTrue: ['HTTP/1.1 '] ifFalse: ['HTTP/1.0 '].
	aSwazooStream
		nextPutAll: version;
		print: self code;
		space;
		nextPutAll: (self class statusTextForCode: self code).
	self crlfOn: aSwazooStream! !

!HTTPResponse methodsFor: 'private-sending' stamp: 'mivsek 12/31/2007 15:54'!
writeHeaderTo: aSwazooStream 
	self printStatusOn: aSwazooStream.
	self printHeadersOn: aSwazooStream.
	(self isStreamed and: [self shouldBeChunked])
		ifTrue: [self printChunkedTransferEncodingOn: aSwazooStream]
		ifFalse: [self printContentLengthOn: aSwazooStream].
	self endHeaderOn: aSwazooStream! !

!HTTPResponse methodsFor: 'sending' stamp: 'janko 7/26/2008 13:49'!
writeTo: aSwazooStream 
	self writeTo: aSwazooStream inResponseTo: nil! !

!HTTPResponse methodsFor: 'sending' stamp: 'janko 7/26/2008 13:48'!
writeTo: aSwazooStream inResponseTo: aRequest
	aSwazooStream isNil ifTrue: [ ^self ].
	self writeHeaderTo: aSwazooStream.
	(aRequest isNil or: [ aRequest isHead not ])
		ifTrue: [ self printEntityOn: aSwazooStream ].
	aSwazooStream closeResponse! !

TestCase subclass: #SwazooBufferTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!SwazooBufferTest methodsFor: 'testing-mime boundary' stamp: 'janko 7/26/2008 15:31'!
testBoundaryFull
	| boundary buffer |
	boundary := '--boundary--'.
	buffer := SwazooBuffer on: 'just--boundary--something'.  "full boundary"
	self assert: (buffer signsOfBoundary: boundary) = boundary size.! !

!SwazooBufferTest methodsFor: 'testing-mime boundary' stamp: 'janko 7/26/2008 15:31'!
testBoundaryMixed
	| boundary buffer |
	boundary := '--boundary--'.
	buffer := SwazooBuffer on: 'yes,--just--boundary--something'.  "partial, later full boundary"
	self assert: (buffer signsOfBoundary: boundary) = boundary size.! !

!SwazooBufferTest methodsFor: 'testing-mime boundary' stamp: 'janko 7/26/2008 15:31'!
testBoundaryOnEdge
	"part of boundary at the end of this buffer, remaining probably in the next"
	| boundary buffer |
	boundary := '--boundary--'.
	buffer := SwazooBuffer on: 'just something-'.  "just first char of boundary"
	self assert: (buffer signsOfBoundary: boundary) = 1.
	buffer := SwazooBuffer on: 'just something--'.  "two chars"
	self assert: (buffer signsOfBoundary: boundary) = 2.
	buffer := SwazooBuffer on: 'just something--bound'.  "half"
	self assert: (buffer signsOfBoundary: boundary) = 7.
	buffer := SwazooBuffer on: 'just something--boundary--'.  "full boundary at the edge"
	self assert: (buffer signsOfBoundary: boundary) = boundary size.! !

!SwazooBufferTest methodsFor: 'testing-mime boundary' stamp: 'janko 7/26/2008 15:32'!
testBoundaryOnEdgeMixed
	"signs of boundary in the middle part at the end of this buffer, remaining probably in the next"
	| boundary buffer |
	boundary := '--boundary--'.
	buffer := SwazooBuffer on: 'just-something-'.  "sign in the middle, one char at the end"
	self assert: (buffer signsOfBoundary: boundary) = 1.
	buffer := SwazooBuffer on: 'just-something--'.  "two chars"
	self assert: (buffer signsOfBoundary: boundary) = 2.
	buffer := SwazooBuffer on: 'just-so--mething--bound'.  "even more mixed case"
	self assert: (buffer signsOfBoundary: boundary) = 7.! !

!SwazooBufferTest methodsFor: 'testing-mime boundary' stamp: 'janko 7/26/2008 15:32'!
testBoundarySimple
	| boundary buffer |
	boundary := '--boundary--'.
	buffer := SwazooBuffer on: 'just something'.  "no boundary"
	self assert: (buffer signsOfBoundary: boundary) = 0.
	buffer := SwazooBuffer on: 'just-something'.  "sign of boundary"
	self assert: (buffer signsOfBoundary: boundary) = 0.
	buffer := SwazooBuffer on: 'just--something'.  "more sign of boundary"
	self assert: (buffer signsOfBoundary: boundary) = 0.
	buffer := SwazooBuffer on: 'just--boundary--something'.  "full boundary"
	self assert: (buffer signsOfBoundary: boundary) = boundary size.! !

!SwazooBufferTest methodsFor: 'testing-mime boundary' stamp: 'janko 7/26/2008 15:32'!
testIndexOfBoundary
	"index of start of boundary in buffer, both full or partial at the edge/end of buffer"
	| boundary buffer |
	boundary := '--boundary--'.
	buffer := SwazooBuffer on: 'just something'.  "no boundary"
	self assert: (buffer indexOfBoundary: boundary) = 0.
	buffer := SwazooBuffer on: 'just--boundary--something-'.  "full boundary"
	self assert: (buffer indexOfBoundary: boundary) = 5.
	buffer := SwazooBuffer on: 'just something--boun'.  "partial boundary at the edge"
	self assert: (buffer indexOfBoundary: boundary) = 15.
	buffer := SwazooBuffer on: 'just something-'.  "partial boundary, one char only"
	self assert: (buffer indexOfBoundary: boundary) = 15.
	buffer := SwazooBuffer on: 'just-som--ething--boun'.  "mixed case with partial at the edge"
	self assert: (buffer indexOfBoundary: boundary) = 17.! !

!HeaderFieldTest methodsFor: 'testing' stamp: 'mivsek 7/19/2007 19:28'!
testContentTypeMultiple
	"   HTTP/1.1 header field values can be folded onto multiple lines if the
   continuation line begins with a space or horizontal tab. All linear
   white space, including folding, has the same semantics as SP. A
   recipient MAY replace any linear white space with a single SP before
   interpreting the field value or forwarding the message downstream.

       LWS            = [CRLF] 1*( SP | HT )"

	| requestStream request field |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET / HTTP/1.1';
		nextPutLine: 'Host: 127.0.0.1';
		nextPutLine: 'Content-Type: text/html; ';
		nextPutLine: ' charset=iso-8859-1';
		crlf.
	request := HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents).
	field := request headers fieldNamed: 'content-type'.
	self assert: field name = 'Content-Type'.
	self assert: field mediaType = 'text/html'.
	self assert: (field transferCodings at: 'charset') = 'iso-8859-1'! !

Object subclass: #HTTPConnection
	instanceVariableNames: 'stream loop server task'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

!HTTPConnection class methodsFor: 'instance creation' stamp: ' 14/4/07 23:09'!
socket: aSocket 
	^self new stream: aSocket stream! !

!HTTPConnection methodsFor: 'serving' stamp: 'mivsek 12/31/2007 16:03'!
getAndDispatchMessages
	"^self
	The HTTPRequest is read from my socket stream.  I then pass this request to my server
	to get a response. "
	self stream anyDataReady  "wait for data and if anything read, proceed"
		ifTrue:
			[self task: (SwazooTask newOn: self).
			self readRequestFor: self task.
			self produceResponseFor: self task.
			self task request wantsConnectionClose ifTrue: [self close].
			self task request isHttp10 ifTrue: [self close] ] "well, we won't complicate here"
		ifFalse: 
			[self keepAliveTimeout ifTrue: [^self close].
			(Delay forMilliseconds: 100) wait. "to finish sending, if any"
			self close].! !

!HTTPConnection methodsFor: 'serving' stamp: 'jm 2/24/2009 09:34'!
interact
	"longer description is below method"
	| interactionBlock |
	interactionBlock := 
		[[	[ [true] whileTrue: 
				[self getAndDispatchMessages.
				Processor yield] 
			]
			on: Error
			do: [:ex | "just ignore"] ]
		ifCurtailed: [ 
			(Delay forMilliseconds: 50) wait. "to finish sending, if any"
			self close] ].
	self server isMultiThreading 
		ifTrue: [self loop: (interactionBlock forkAt: Processor userBackgroundPriority)]
		ifFalse: [interactionBlock value].
	^self

"I represent a specifc connection with an HTTP client (a browser, probably) over which will come an HTTP request.  Here, I fork the handling of the request so that the current thread (which is most likely the HTTP server main loop) can carry on with the next request.  This means that more than one request may being handled in the image at a time, and that means that the application developer must worry about thread safety - e.g the problem of a given business object being updated by more than one HTTP request thread.
For a GemStone implementation of Swazoo, one may want only one request is handled at a time, multi-threadedness being handled by having multiple gems.  This is a nice option because the application developer does not have to worry about thread safety in this case - GemStone handles the hard stuff.
*And* the thing called a loop that was in this method was no such thing.  In all circumstances, >>getAndDispatchMessages handles exactly one requst and then closes the socket!! (very non-HTTP1.1).  Anyway, for now I'm just going to make that explicit.  This needs to be re-visited to support HTTP 1.1."! !

!HTTPConnection methodsFor: 'testing' stamp: 'janko 7/26/2008 13:21'!
keepAliveTimeout
	| seconds |
	self task isNil ifTrue: [^false].
	self task request isKeepAlive ifFalse: [^false].
	seconds := self task request keepAlive notNil
		ifTrue: [self task request keepAlive asInteger - 10 "to be sure"]
		ifFalse: [20]. "probably enough?"
	^(SpTimestamp now asSeconds - self task request timestamp asSeconds) >= seconds! !

!HTTPConnection methodsFor: 'serving-responses' stamp: ' 14/8/07 21:49'!
nextPutError: aResponse 
	aResponse informConnectionClose.
	aResponse writeTo: self stream.
	self stream closeResponse.! !

!HTTPConnection methodsFor: 'serving-responses' stamp: 'mivsek 12/31/2007 16:04'!
nextPutResponse: aMessage toRequest: aRequest 
	aRequest isHead 
		ifTrue: [aMessage writeHeaderTo: self stream]
		ifFalse: [aMessage writeTo: self stream].
	self stream closeResponse.! !

!HTTPConnection methodsFor: 'serving' stamp: 'mivsek 12/31/2007 16:06'!
readRequestFor: aSwazooTask 
	"I read the next request from my socket and add it to aSwazooTask.  If I have any 
	problems and need to force a bad request (400) response, I add this response to aSwazooTask."
	| request |
	SpExceptionContext for: 
		[request := HTTPRequest readFrom: self stream.
		request uri port: self server port.
		(request httpVersion last = 1 
			and: [(request headers includesFieldOfClass: HTTPHostField) not]) 
				ifTrue: [aSwazooTask response: HTTPResponse badRequest].
		[request peer: self stream socket remoteAddress]
			on: Error do: [:ex | "do nothing for now"].
		request
			ip: self stream socket localAddress;
			setTimestamp.
		aSwazooTask request: request]
	on: SpError, HTTPException
	do: [:ex | 
		aSwazooTask response: HTTPResponse badRequest.
		self nextPutError: aSwazooTask response.
		self close].! !

!HTTPConnection methodsFor: 'private' stamp: ' 14/4/07 23:09'!
stream: aSwazooStream 
	stream := aSwazooStream! !

Object subclass: #HTTPServer
	instanceVariableNames: 'ip port connections sites socket loop isMultiThreading'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
acceptConnection
	"^self
I accept the next inbound TCP/IP connection.  The operating system libraries queue these up for me, so I can just handle one at a time.  I create an HTTPConnection instance to actually handle the interaction with the client - if I am in single threaded mode, the connection will completely handle the request before returning control to me, but in multi-threaded mode the connection forks the work into a sepparate thread in this image and control is immediately returned to me (the application programmer must worry about thread safety in this case."

	| clientConnection |
	clientConnection := SpExceptionContext 
				for: [HTTPConnection socket: self socket accept]
				on: SpError
				do: [:ex | Transcript show: 'Socket accept error: ' , ex errorString; cr. ^self].
	self addConnection: clientConnection.
	clientConnection interact.
	^self! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
addConnection: aConnection 
	self connections add: aConnection.
	aConnection server: self! !

!HTTPServer methodsFor: 'sites' stamp: ' 14/4/07 23:09'!
addSite: aSite
	(self sites includesResource: aSite) ifFalse: [^self sites addResource: aSite]! !

!HTTPServer methodsFor: 'serving' stamp: ' 14/4/07 23:09'!
answerTo: aRequest 
	| response |
	response := URIResolution resolveRequest: aRequest startingAt: self sites.
	^response isNil
		ifTrue: [HTTPResponse notFound]
		ifFalse: [response]! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
connections
	connections isNil ifTrue: [self initConnections].
	^connections! !

!HTTPServer methodsFor: 'sites' stamp: ' 14/4/07 23:09'!
hasNoSites
	^self sites hasNoResources! !

!HTTPServer methodsFor: 'private-initialize' stamp: ' 14/4/07 23:09'!
initConnections
	connections := OrderedCollection new.! !

!HTTPServer methodsFor: 'private-initialize' stamp: ' 14/4/07 23:09'!
initSites
	sites := ServerRootComposite new! !

!HTTPServer methodsFor: 'private-initialize' stamp: ' 14/4/07 23:09'!
initialize
	self initConnections.
	self initSites! !

!HTTPServer methodsFor: 'multithreading' stamp: ' 14/4/07 23:09'!
isMultiThreading
	"^a Boolean
I return true if each inbound HTTP connection will be handled in its own thread.  See the senders of this message to see where that is important.  Note that the default mode is mult-threaded because this is how Swazoo has worked so far.  This is tricky for the application programmer, though, as they must ensure that they work in a thread safe way (e.g. avoid the many threads updating the same object).  For those deploying to GemStone, you wil find things much easier if you do *not* run multithreaded, but rather run many gems each with a single-threaded Swazoo instance (and your app logic) in each.  Also in GemStone, run the main loop in the foreground, c.f. >>mainLoopInForeground"

	isMultiThreading isNil ifTrue: [self setMultiThreading].
	^isMultiThreading! !

!HTTPServer methodsFor: 'testing' stamp: ' 14/4/07 23:09'!
isServing
	^self loop notNil! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
loop
	^loop! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
loop: aProcess
	loop := aProcess! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
removeConnection: aConnection 
	self connections remove: aConnection ifAbsent: [nil]! !

!HTTPServer methodsFor: 'sites' stamp: ' 14/4/07 23:09'!
removeSite: aSite 
	^self sites removeResource: aSite! !

!HTTPServer methodsFor: 'start/stop' stamp: 'mivsek 11/15/2007 13:51'!
restart
	"usefull after image startup, when socket is probably not valid anymore"
	self stop.
	self start.! !

!HTTPServer methodsFor: 'multithreading' stamp: ' 14/4/07 23:09'!
setMultiThreading
	"^self
I record that this HTTP server is to operate in a multi-threaded mode.  c.f. isMultiThreading"

	isMultiThreading := true.
	^self! !

!HTTPServer methodsFor: 'multithreading' stamp: ' 14/4/07 23:09'!
setSingleThreading
	"^self
I record that this HTTP server is to operate in a single-threaded mode.  c.f. isMultiThreading"

	isMultiThreading := false.
	^self! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
sites
	sites isNil ifTrue: [self initSites].
	^sites! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
socket
	^socket! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
socket: aSocket
	socket := aSocket! !

!HTTPServer methodsFor: 'start/stop' stamp: 'mivsek 11/15/2007 13:51'!
start
	self loop isNil 
		ifTrue: 
			[self socket: (self socketClass serverOnIP: self ipCorrected port: self port).
			self socket listenFor: 50.
			self loop: ([[self acceptConnection] repeat] 
						forkAt: Processor userBackgroundPriority)]! !

!HTTPServer methodsFor: 'start/stop' stamp: ' 14/4/07 23:09'!
stop
	self loop isNil 
		ifFalse: 
			[self connections copy do: [:each | each close].
			self loop terminate.
			self loop: nil.
			self socket close.
			self socket: nil]! !

Object subclass: #SwazooServer
	instanceVariableNames: 'sites servers watchdog'
	classVariableNames: 'Singleton'
	poolDictionaries: ''
	category: 'Swazoo-Core'!

!SwazooServer commentStamp: 'mivsek 12/31/2007 16:37' prior: 0!
SwazooServer is where all begins in Swazoo!!
SwazooServer singleton : return one and only one server which holds the Sites. Also used to start and stop all sites ato once, to add new sited etc. When running, a collection of HTTPServers is also stored in SwazooServer singleton.

SwazooServer demoStart  will create and run a demo site on http://localhost:8888 which 
                              returns a web page with 'Hello World!!'!

!SwazooServer class methodsFor: 'private' stamp: 'JM 11/15/2007 19:15'!
initSingleton
	Singleton := super new! !

!SwazooServer methodsFor: 'initialize-release' stamp: ' 14/4/07 23:09'!
initialize
	self initSites.
	self initServers.! !

Object subclass: #SwazooSocket
	instanceVariableNames: 'accessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

!SwazooSocket methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
read: anInteger timeout: aNumberOfMilliseconds 
	^(self accessor waitForReadDataUpToMs: aNumberOfMilliseconds) 
		ifTrue: [self read: anInteger]
		ifFalse: [ByteArray new]! !

!SwazooSocket methodsFor: 'private' stamp: ' 14/4/07 23:09'!
stream
	^SwazooStream socket: self! !

Object subclass: #SwazooStream
	instanceVariableNames: 'socket readBuffer writeBuffer chunked'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

!SwazooStream class methodsFor: 'instance creation' stamp: ' 14/4/07 23:09'!
connectedPair
	^SwazooSocket connectedPair collect: [:each | self socket: each]! !

!SwazooStream class methodsFor: 'instance creation' stamp: 'janko 7/26/2008 18:31'!
on: aString 
	"use only for testing!!"
	^self new readBuffer: (SwazooBuffer on: aString)! !

!SwazooStream class methodsFor: 'instance creation' stamp: ' 14/4/07 23:09'!
socket: aSwazooSocket 
	^self new setSocket: aSwazooSocket! !

!SwazooStream methodsFor: 'accessing-reading' stamp: 'janko 7/26/2008 18:33'!
anyDataReady
	"wait for data and return true if any data ready. On VW somethimes happen that data 
      receipt is signaled but no data is actually received"
	self readBuffer atEnd ifFalse: [^true]. "data is there from before"
	self fillBuffer.
	^self readBuffer atEnd not! !

!SwazooStream methodsFor: 'accessing-reading' stamp: ' 14/8/07 21:49'!
atEnd
	"TCP socket data never ends!!"
	^false! !

!SwazooStream methodsFor: 'initialize-release' stamp: 'janko 7/26/2008 18:33'!
close
	"close TCP socket and relase buffers"
	self socket close.
	self nilWriteBuffer.
	self nilReadBuffer.  "to GC buffers"! !

!SwazooStream methodsFor: 'initialize-release' stamp: 'janko 7/26/2008 18:34'!
closeResponse
	"for chunked response: close it by sending null chunk"
	"do a bit cleanup after response is sent"
	self flush.
	self isChunked ifTrue: 
		[self writeBuffer closeChunkTo: self socket.
		self resetChunked].
	self socket notNil "not simulation" ifTrue: [self nilWriteBuffer]. "to release memory"! !

!SwazooStream methodsFor: 'accessing-writing' stamp: ' 14/8/07 21:49'!
cr
	self nextPut: Character cr asInteger! !

!SwazooStream methodsFor: 'accessing-writing' stamp: 'janko 7/26/2008 18:34'!
crlf
	self cr; lf.! !

!SwazooStream methodsFor: 'private' stamp: 'janko 7/26/2008 18:34'!
fillBuffer
	self socket isNil ifTrue: [^self]. "if SwazooStream is used for tests only"
	self readBuffer refillFrom: self socket.! !

!SwazooStream methodsFor: 'initialize-release' stamp: 'janko 5/7/2008 16:57'!
flush
	"actually write to the tcp socket and clear write buffer"
	self socket isNil ifTrue: [^nil]. "for simulations and tests"
	self isChunked
		ifTrue: [self writeBuffer flushChunkTo: self socket]
		ifFalse: [	self writeBuffer flushTo: self socket].! !

!SwazooStream methodsFor: 'initialize-release' stamp: 'janko 7/26/2008 18:34'!
initReadBuffer
	"temporary read buffer"
	readBuffer := SwazooBuffer newRead! !

!SwazooStream methodsFor: 'initialize-release' stamp: 'janko 7/26/2008 18:35'!
initWriteBuffer
	"temporary write buffer. flush it to socket ocassionaly!!"
	writeBuffer := SwazooBuffer newWrite! !

!SwazooStream methodsFor: 'chunking' stamp: ' 14/8/07 21:49'!
isChunked
	"sending in chunks (transfer encoding: chunked)"
	^chunked notNil and: [chunked]! !

!SwazooStream methodsFor: 'accessing-writing' stamp: ' 14/8/07 21:49'!
lf
	self nextPut: Character lf asInteger! !

!SwazooStream methodsFor: 'accessing-reading' stamp: ' 14/8/07 21:49'!
next
	^self nextByte asCharacter! !

!SwazooStream methodsFor: 'accessing-reading' stamp: 'janko 7/26/2008 18:51'!
next: anInteger 
	^(self nextBytes: anInteger) asString! !

!SwazooStream methodsFor: 'private-stream' stamp: ' 14/8/07 21:49'!
nextByte
	self syncBuffer.
	^self readBuffer next.
! !

!SwazooStream methodsFor: 'private-stream' stamp: 'janko 7/26/2008 18:36'!
nextBytes: aNumber 
	| array |
	array := ByteArray new: aNumber.
	1 to: aNumber do: [:index |  array at: index put: self nextByte].
	^array! !

!SwazooStream methodsFor: 'accessing-reading' stamp: 'janko 7/26/2008 18:36'!
nextLine
	| stream |
	stream := WriteStream on: (String new: 50).
	self writeNextLineTo: stream.
	^stream contents! !

!SwazooStream methodsFor: 'accessing-writing' stamp: ' 14/8/07 21:49'!
nextPut: aCharacterOrInteger
	self nextPutByte: aCharacterOrInteger asInteger.
	^aCharacterOrInteger! !

!SwazooStream methodsFor: 'accessing-writing' stamp: ' 14/8/07 21:49'!
nextPutAll: aByteStringOrArray
	self nextPutBytes: aByteStringOrArray asByteArray.
	^aByteStringOrArray! !

!SwazooStream methodsFor: 'private-stream' stamp: ' 14/8/07 21:49'!
nextPutByte: aByte 
	self writeBuffer nextPut: aByte.
	self writeBuffer isFull ifTrue: [self flush].
	^aByte! !

!SwazooStream methodsFor: 'private-stream' stamp: ' 14/8/07 21:49'!
nextPutBytes: aByteArray 
	self writeBuffer nextPutAll: aByteArray.
	self writeBuffer isFull ifTrue: [self flush]. 
	^aByteArray! !

!SwazooStream methodsFor: 'accessing-writing' stamp: 'janko 7/26/2008 18:36'!
nextPutLine: aByteStringOrArray 
	self nextPutAll: aByteStringOrArray.
	self crlf.! !

!SwazooStream methodsFor: 'accessing-reading' stamp: 'janko 7/26/2008 18:37'!
nextUnfoldedLine
	| stream ch |
	stream := WriteStream on: (String new: 50).
	self writeNextLineTo: stream.
	stream contents isEmpty ifFalse: [
		[ch := self peek.
		ch notNil and: [ch == Character space or: [ch == Character tab]]] 
			whileTrue: [self writeNextLineTo: stream]
		].
	^stream contents! !

!SwazooStream methodsFor: 'initialize-release' stamp: 'janko 7/26/2008 18:37'!
nilReadBuffer
	"to release memory"
	readBuffer := nil! !

!SwazooStream methodsFor: 'initialize-release' stamp: ' 14/8/07 21:49'!
nilWriteBuffer
	"to release memory"
	writeBuffer := nil! !

!SwazooStream methodsFor: 'accessing-reading' stamp: ' 14/8/07 21:49'!
peek
	| byte |
	byte := self peekByte.
	^byte notNil 
		ifTrue: [byte asCharacter] 
		ifFalse: [nil]! !

!SwazooStream methodsFor: 'private-stream' stamp: ' 14/8/07 21:49'!
peekByte
	self syncBuffer.
	^self readBuffer peek! !

!SwazooStream methodsFor: 'private' stamp: ' 14/8/07 21:49'!
print: anObject 
	anObject printOn: self! !

!SwazooStream methodsFor: 'private' stamp: 'janko 7/26/2008 18:37'!
readBuffer
	readBuffer isNil ifTrue: [self initReadBuffer].
	^readBuffer! !

!SwazooStream methodsFor: 'private' stamp: 'janko 7/26/2008 18:38'!
readBuffer: aSwazooBuffer
	readBuffer := aSwazooBuffer! !

!SwazooStream methodsFor: 'chunking' stamp: ' 14/8/07 21:49'!
resetChunked
	"sending in chunks (transfer encoding: chunked)"
	chunked := false! !

!SwazooStream methodsFor: 'chunking' stamp: ' 14/8/07 21:49'!
setChunked
	"sending in chunks (transfer encoding: chunked)"
	chunked := true! !

!SwazooStream methodsFor: 'private' stamp: 'janko 7/26/2008 18:38'!
setSocket: aSwazooSocket 
	self socket: aSwazooSocket.! !

!SwazooStream methodsFor: 'accessing-reading' stamp: 'janko 7/26/2008 18:39'!
skip: anInteger
	anInteger timesRepeat: [self nextByte].! !

!SwazooStream methodsFor: 'private' stamp: ' 14/8/07 21:49'!
socket
	^socket! !

!SwazooStream methodsFor: 'private' stamp: ' 14/4/07 23:09'!
socket: aSocket
	socket := aSocket! !

!SwazooStream methodsFor: 'accessing-writing' stamp: ' 14/8/07 21:49'!
space
	self nextPut: Character space! !

!SwazooStream methodsFor: 'private' stamp: ' 14/4/07 23:09'!
syncBuffer
	self readBuffer atEnd ifTrue: [self fillBuffer]! !

!SwazooStream methodsFor: 'accessing-reading' stamp: ' 14/8/07 21:49'!
upTo: aCharacterOrByte
	| targetByte ws byte |
	targetByte := aCharacterOrByte asInteger.
	ws := ByteArray new writeStream.
	[byte := self nextByte.
	byte = targetByte] whileFalse: [ws nextPut: byte].
	^ws contents! !

!SwazooStream methodsFor: 'private' stamp: ' 10/7/07 17:21'!
writeBuffer
	writeBuffer isNil ifTrue: [self initWriteBuffer].
	^writeBuffer! !

!SwazooStream methodsFor: 'accessing-reading' stamp: 'mivsek 8/14/2007 22:41'!
writeNextLineTo: aStream 
	aStream nextPutAll: (self upTo: Character cr asInteger) asString.
	^self peekByte = Character lf asInteger 
		ifTrue: [self nextByte]  "skip remaining linefeed"
		ifFalse: [SwazooHTTPParseError raiseSignal: 'CR without LF']! !

!SwazooStreamTest methodsFor: 'testing-lines' stamp: 'mivsek 7/19/2007 19:27'!
testLinesWithDoubleCRLF
	| ws rs comparisonString |
	comparisonString := 'abcd'.
	ws := SwazooStream on: String new.
	ws nextPutAll: comparisonString.
	self crlfOn: ws.
	self crlfOn: ws.
	rs := SwazooStream on: ws writeBuffer contents.
	self assert: rs nextLine = comparisonString.
	self assert: rs nextLine = ''! !

!SwazooStreamTest methodsFor: 'testing-lines' stamp: 'mivsek 7/19/2007 19:27'!
testSingleLineWithCR
	| ws rs comparisonString errored |
	comparisonString := 'abcd' , (String with: Character cr) , 'efg'.
	ws := SwazooStream on: String new.
	ws nextPutAll: comparisonString.
	ws nextPut: Character cr.
	rs := SwazooStream on: ws writeBuffer contents.
	errored := false.
	SpExceptionContext 
		for: [rs nextLine]
		on: SpError
		do: [:ex | errored := true].
	self assert: errored! !

!SwazooStreamTest methodsFor: 'testing-lines' stamp: 'mivsek 7/19/2007 19:27'!
testSingleLineWithCRLF
	| ws rs comparisonString |
	comparisonString := 'abcd'.
	ws := SwazooStream on: String new.
	ws nextPutAll: comparisonString.
	self crlfOn: ws.
	rs := SwazooStream on: ws writeBuffer contents.
	self assert: rs nextLine = comparisonString! !

!SwazooServerTest methodsFor: 'testing' stamp: ' 8/2/09 22:22'!
testStartingOnTwoPorts
	"and all ip interfaces, any host"
	
	| server nrServers site1 site2 |
	server _ SwazooServer singleton.
	nrServers _ server servers size.
	[site1 _ server startOn: 4924.
	site2 _ server startOn: 4925.
	self assert: site1 isServing.
	self assert: site2 isServing.
	self assert: server servers size = (nrServers + 2).
	server stopOn: 4924.
	server stopOn: 4925.
	self assert: site1 isServing not.
	self assert: site2 isServing not.
	self assert: server servers size = nrServers]
		ensure:
			[site1 stop.
			site2 stop.
			server
				removeSite: site1;
				removeSite: site2]! !

!HTTPRequestLine class methodsFor: 'instance creation' stamp: 'janko 7/26/2008 18:42'!
readFrom: aSwazooStream 
	^self new readFrom: aSwazooStream! !

!HTTPRequestLine methodsFor: 'private' stamp: ' 14/4/07 23:10'!
httpVersion: anArray 
	httpVersion := anArray.
	^self! !

!HTTPRequestLine methodsFor: 'private' stamp: ' 14/4/07 23:10'!
method: aString
"For development testing only"
	method := aString.
	^self! !

!HTTPRequestLine methodsFor: 'parsing' stamp: 'mivsek 7/26/2007 13:07'!
parseHTTPVersionFrom: aSwazooStream 
	| major minor |
	self skipSpacesIn: aSwazooStream.
	aSwazooStream upTo: $/ asInteger.
	major := (aSwazooStream upTo: $. asInteger) asString asNumber.
	minor := (aSwazooStream upTo: Character cr asInteger) asString asNumber.
	self httpVersion: (Array with: major with: minor).
	aSwazooStream next.
	^self! !

!HTTPRequestLine methodsFor: 'parsing' stamp: 'mivsek 7/26/2007 13:06'!
parseURIFrom: aSwazooStream 
	"^self
	Really, we should parse the URI directly out of the stream."

	self skipSpacesIn: aSwazooStream.
	requestURI := SwazooURI fromString: 
		(aSwazooStream upTo: Character space asInteger) asString.
	^self! !

!HTTPRequestLine methodsFor: 'parsing' stamp: 'mivsek 7/10/2007 18:52'!
readFrom: aSwazooStream 
	"^self
I initialize myself to represents a request line read from aStream.  If no valid request line can be found, I throw an exception."

	self skipLeadingBlankLinesIn: aSwazooStream.
	method := (aSwazooStream upTo: Character space asInteger) asString.
	self parseURIFrom: aSwazooStream.
	self parseHTTPVersionFrom: aSwazooStream.
	^self! !

!HTTPRequestLine methodsFor: 'private' stamp: ' 14/4/07 23:10'!
requestURI: aString 
	"Development testing only!!"

	requestURI := aString.
	^self! !

!HTTPRequestLine methodsFor: 'private' stamp: ' 10/7/07 17:21'!
skipLeadingBlankLinesIn: aSwazooStream 
	"^self
RFC 2616:
In the interest of robustness, servers SHOULD ignore any empty
line(s) received where a Request-Line is expected. In other words, if
the server is reading the protocol stream at the beginning of a
message and receives a CRLF first, it should ignore the CRLF."

	[aSwazooStream peek == Character cr asInteger] whileTrue: 
			[(((aSwazooStream next: 2) at: 2) == Character lf asInteger) 
				ifFalse: [SwazooHTTPParseError raiseSignal: 'CR without LF']].
	^self! !

!HTTPRequestLine methodsFor: 'private' stamp: 'mivsek 7/26/2007 13:27'!
skipSpacesIn: aSwazooStream 
	[aSwazooStream peek = Character space] 
		whileTrue: [aSwazooStream next].
	^self! !

!FileResponse methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
entity: aMimeObject 
	entity := aMimeObject! !

!FileResponse methodsFor: 'private-printing' stamp: ' 14/4/07 23:10'!
printContentLengthOn: aStream 
	self entity isNil 
		ifFalse: 
			[aStream
				nextPutAll: 'Content-Length: ';
				print: self entity value fileSize.
			self crlfOn: aStream]! !

!FileResponse methodsFor: 'private-printing' stamp: ' 14/4/07 23:10'!
printHeadersOn: aStream 
	self contentType: self entity contentType.
	super printHeadersOn: aStream! !

!HTTPResponseTest methodsFor: 'testing' stamp: 'mivsek 7/19/2007 19:28'!
testInternalServerError
	| ws rs |
	response := HTTPResponse internalServerError.
	ws := SwazooStream on: String new.
	response printStatusOn: ws.
	rs := SwazooStream on: ws writeBuffer contents.
	self assert: rs nextLine = 'HTTP/1.1 500 Internal Server Error'! !

!HTTPResponseTest methodsFor: 'testing' stamp: 'mivsek 7/19/2007 19:28'!
testOK
	| ws rs |
	response := HTTPResponse ok.
	ws := SwazooStream on: String new.
	response printStatusOn: ws.
	rs := SwazooStream on: ws writeBuffer contents.
	self assert: rs nextLine = 'HTTP/1.1 200 OK'! !

!HTTPHeaders class methodsFor: 'instance creation' stamp: ' 10/7/07 17:21'!
readFrom: aSwazooStream 
	"^an HTTPHeaders
I return a new instance of myself which contains fields parsed from aStream."

	^self new readFrom: aSwazooStream! !

!HTTPHeaders methodsFor: 'initialize-release' stamp: ' 14/4/07 23:10'!
readFieldFromString: aString 
	"^self
First I get the field parsed from aString, then I add the new field to my collection of fields.  Adding the new field may involve merging field values if I already have a field of that class."

	self addField: (HeaderField fromLine: aString).
	^self! !

!HTTPHeaders methodsFor: 'initialize-release' stamp: ' 10/7/07 17:21'!
readFrom: aSwazooStream 
	"^an HTTPHeaders
I return a new instance of myself which contains fields parsed from aStream.  Everything upto the next blank line is a header field."
	
	| nextLine |
	[nextLine := aSwazooStream nextUnfoldedLine.
	nextLine isEmpty]
		whileFalse: [self readFieldFromString: nextLine].
	^self! !

!HTTPHeaders methodsFor: 'emitting' stamp: ' 14/4/07 23:10'!
writeOn: aStream 
	"^self
I write all my fields to aStream."

	self fields do: 
			[:aField | 
			aField printOn: aStream.
			self crlfOn: aStream]! !

!HTTPRequest class methodsFor: 'instance creation' stamp: ' 10/7/07 17:21'!
newFor: aRequestLine readFrom: aSwazooStream 
	"to support an additional http method, simply subclass a HTTPRequest!!"
	| targetClass |
	targetClass := aRequestLine method = 'GET' 
		ifTrue: [HTTPGet] "most used anyway"
		ifFalse: [aRequestLine method = 'POST'  
			ifTrue: [HTTPPost]  "second most used"
			ifFalse: [self subclasses detect: [:each | 
				each methodName = aRequestLine method] ifNone: [nil] ] ].
	targetClass isNil ifTrue: [^HTTPException notImplemented].
	^targetClass new for: aRequestLine readFrom: aSwazooStream! !

!HTTPRequest class methodsFor: 'instance creation' stamp: ' 10/7/07 17:21'!
readFrom: aSwazooStream 
	"^an HTTPRequest
I create and return a new instance of one of my subclasses which will represent the HTTP request presumed to be the contents of aStream.
The first step is to work out which of my subclasses to create.  I do this by parsing the 'request-line' from the stream.  The request-line contains the 'method', and I look for the subclass that handles this method and delegate the rest of the message parsing to a new instance of that class."

	| requestLine |
	requestLine := HTTPRequestLine readFrom: aSwazooStream.
	^self newFor: requestLine readFrom: aSwazooStream! !

!HTTPRequest methodsFor: 'initialize-release' stamp: ' 10/7/07 17:21'!
for: aRequestLine readFrom: aSwazooStream 
	"^self
I parse my headers from aStream and update my URI and HTTP version information from aRequest line.  I need to parse the headers first because, for some reason, the URI insists on knowing the host, and this is taken from the Host: header field."

	requestLine := aRequestLine.
	headers := HTTPHeaders readFrom: aSwazooStream.
	self setTimestamp.
	^self! !

!HTTPRequest methodsFor: 'testing' stamp: 'mivsek 12/31/2007 15:44'!
isKeepAlive
	| header |
	header := self connection.
	header isNil ifTrue: [^false].
	^'*Keep-Alive*' match: header! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: 'mivsek 12/31/2007 15:44'!
keepAlive
	"how many seconds a connection must be kept alive"
	^(self headers fieldNamed: 'KeepAlive' ifNone: [^nil]) value! !

!HTTPPostTest methodsFor: 'requests' stamp: 'janko 7/26/2008 15:33'!
fileContents
	"HTTPRequestTest new fileContents"
	| stream |
	stream := SwazooStream on: String new.
	stream
		nextPutLine: 'BEGIN:VCALENDAR';
		nextPutLine: 'PRODID:-//Squeak-iCalendar//-';
		nextPutLine: 'VERSION:2.0';
		nextPutLine: 'X-WR-CALNAME:test';
		nextPutLine: 'METHOD:PUBLISH';
		nextPutLine: 'BEGIN:VEVENT';
		nextPutLine: 'UID:an event with a start date and not date and time';
		nextPutLine: 'CATEGORIES:category1,category2';
		nextPutLine: 'CREATED:20050501T110231Z';
		nextPutLine: 'SEQUENCE:0';
		nextPutLine: 'SUMMARY:aTitle';
		nextPutLine: 'PRIORITY:5';
		nextPutLine: 'DTSTART;VALUE=DATE:20050425';
		nextPutLine: 'END:VEVENT';
		nextPutLine: 'END:VCALENDAR'.

	^stream writeBuffer contents asString! !

!HTTPPostTest methodsFor: 'requests' stamp: 'janko 7/26/2008 15:33'!
postDashes
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST  /document/aab.html HTTP/1.1';
		nextPutLine: 'Host: biart.eranova.si';
		nextPutLine: 'Content-Type: multipart/form-data; boundary= boundary';
		nextPutLine: 'Content-Length: 149';
            crlf;
		nextPutLine: '--boundary';
		nextPutLine: 'Content-Disposition: form-data; name="id5273"';
		crlf; 
		nextPutLine: '----';
		nextPutLine: '--boundary';
		nextPutLine: 'Content-Disposition: form-data; name="field2"';
		crlf; 
		nextPutLine: '- --';
		nextPutLine: '--boundary--'.

	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPPostTest methodsFor: 'requests' stamp: 'janko 7/26/2008 15:33'!
postEmpty
	"post entity with empty value"
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST  /document/aab.html HTTP/1.1';
		nextPutLine: 'Host: biart.eranova.si';
		nextPutLine: 'Content-Type: multipart/form-data; boundary= boundary';
		nextPutLine: 'Content-Length: 75';
            crlf;
		nextPutLine: '--boundary';
		nextPutLine: 'Content-Disposition: form-data; name="id5273"';
		crlf; 
		nextPutLine: '';
		nextPutLine: '--boundary--'.

	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPPostTest methodsFor: 'requests' stamp: 'janko 7/26/2008 15:33'!
postFile
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST  /document/aab.html HTTP/1.1';
		nextPutLine: 'Connection: Keep-Alive';
		nextPutLine: 'User-Agent: Mozilla/4.72 [en] (X11; I; Linux 2.3.51 i686)';
		nextPutLine: 'Host: biart.eranova.si';
		nextPutLine: 'Referer: http://www.bar.com/takeMeThere.html';
		nextPutLine: 'Content-Type: multipart/form-data; boundary= -----------------20752836116568320241700153999';
		nextPutLine: 'Content-Length: ', (527+self fileContents size) printString;
            crlf;
		nextPutLine: '-------------------20752836116568320241700153999';
		nextPutLine: 'Content-Disposition: form-data; name="id5273"';
		crlf; 
		nextPutLine: 'main';
		nextPutLine: '-------------------20752836116568320241700153999';
		nextPutLine: 'Content-Disposition: form-data; name="field2"';
		crlf; crlf;
		nextPutLine: '-------------------20752836116568320241700153999';
		nextPutLine: 'Content-Disposition: form-data; name="field7"; filename="event.ical"';
		nextPutLine: 'Content-Type: application/octet-stream';
		crlf;

		nextPutAll: self fileContents;

		crlf;
		nextPutLine: '-------------------20752836116568320241700153999';
		nextPutLine: 'Content-Disposition: form-data; name="attach"';
		crlf;
		nextPutLine: 'Attach';
		nextPutLine: '-------------------20752836116568320241700153999--'.

	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPPostTest methodsFor: 'requests' stamp: 'janko 7/26/2008 15:33'!
postPreambleEpilogue
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST  /document/aab.html HTTP/1.1';
		nextPutLine: 'Host: biart.eranova.si';
		nextPutLine: 'Content-Type: multipart/form-data; boundary= boundary';
		nextPutLine: 'Content-Length: 146';
            crlf;
		nextPutLine: 'This is a multi-part message in MIME format';
		nextPutLine: '--boundary';
		nextPutLine: 'Content-Disposition: form-data; name="id5273"';
		crlf; 
		nextPutLine: 'main';
		nextPutLine: '--boundary--';
		nextPutLine: 'This is the epilogue'.

	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPPostTest methodsFor: 'requests' stamp: 'janko 7/26/2008 15:33'!
postSimple
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST  /document/aab.html HTTP/1.1';
		nextPutLine: 'Host: biart.eranova.si';
		nextPutLine: 'Content-Type: multipart/form-data; boundary= boundary';
		nextPutLine: 'Content-Length: 79';
            crlf;
		nextPutLine: '--boundary';
		nextPutLine: 'Content-Disposition: form-data; name="id5273"';
		crlf; 
		nextPutLine: 'main';
		nextPutLine: '--boundary--'.

	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPPostTest methodsFor: 'requests' stamp: 'janko 7/26/2008 15:33'!
postUrlEncoded
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST  /document/aab.html HTTP/1.1';
		nextPutLine: 'Host: biart.eranova.si';
		nextPutLine: 'Content-Type: application/x-www-form-urlencoded';
		nextPutLine: 'Content-Length: 36';
            crlf;
		nextPutAll: 'home=Cosby+one&favorite+flavor=flies'.

	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPPostTest methodsFor: 'testing-mime parsing' stamp: 'janko 7/26/2008 15:35'!
testBlockCopy
	"streaming with 8k blocks for performance"
	"this is just a basic test with content shorter that one block"
	| boundary message in out |
	boundary := '--boundary--'.
	message := 'just something'. 
	in := SwazooStream on: message, self crlf, boundary.
	out := WriteStream on: String new.
	HTTPPost new blockStreamingFrom: in to: out until: boundary.
	self assert: out contents = message.! !

!HTTPPostTest methodsFor: 'testing-posts' stamp: 'janko 7/26/2008 15:33'!
testPostRawEntity
	| requestStream post |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST /foobar HTTP/1.0';
		nextPutLine: 'Host: foo.com';
		nextPutLine: 'Content-Type: text/plain';
		nextPutLine: 'Content-Length: 12';
		crlf;
		nextPutLine: 'Hello, World'.
	post := HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents).
	self assert: post isPostDataEmpty.
	self assert: post entityBody = 'Hello, World'! !

!HTTPPostTest methodsFor: 'testing-posts' stamp: 'janko 7/26/2008 15:33'!
testPostUrlEncodedData
	| requestStream post |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST / HTTP/1.1';
		nextPutLine: 'Host: foo.com';
		nextPutLine: 'Content-Type: application/x-www-form-urlencoded';
		nextPutLine: 'Content-Length: 31';
		crlf;
		nextPutLine: 'address=+fs&product=&quantity=1'.
	post := HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents).
	self assert: (post postDataAt: 'address') value = ' fs'.
	self assert: (post postDataAt: 'product') value = ''.
	self assert: (post postDataAt: 'quantity') value = '1'! !

ReadWriteStream subclass: #SwazooBuffer
	instanceVariableNames: 'type resize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

!SwazooBuffer commentStamp: 'janko 7/26/2008 18:23' prior: 0!
SwazooBuffer is used for efficient buffering of receiving or sending data to TCP socket. Efficiency is achieved with reusing of stream content array instead of initializing it everytime buffer is emptied, as was in previous Swazoo versions. 

SwazooBuffer is a subclass of ReadWriteStream, with additional #clear to empty buffer.

!

!SwazooBuffer class methodsFor: 'defaults' stamp: 'janko 7/26/2008 18:25'!
defaultBufferSize
	"length of buffer at creation. Later not nessesary exactly this number!! "
	^8000 "about 8KB-preamble"
! !

!SwazooBuffer class methodsFor: 'defaults' stamp: 'janko 8/3/2008 19:55'!
largeBufferSize
	"size of buffer for large uploads/downloads"
	^1000000	"about 1MB"! !

!SwazooBuffer class methodsFor: 'instance creation' stamp: 'jm 2/8/2009 14:47'!
newRead
	^(super on: Array new) setRead; initDefaultBuffer! !

!SwazooBuffer class methodsFor: 'instance creation' stamp: 'jm 2/8/2009 14:46'!
newWrite
	^(super on: Array new) setWrite;  initDefaultBuffer! !

!SwazooBuffer class methodsFor: 'instance creation' stamp: 'janko 7/26/2008 18:24'!
on: aByteStringOrArray
	^(self with: aByteStringOrArray) position: 0! !

!SwazooBuffer class methodsFor: 'defaults' stamp: ' 14/8/07 21:49'!
preambleSize
	"chunk size line (fixed to 4 hex characters) + crlf"
	^6! !

!SwazooBuffer methodsFor: 'testing' stamp: 'janko 7/26/2008 18:22'!
atEnd
	^super atEnd "for now"! !

!SwazooBuffer methodsFor: 'writing-chunked' stamp: ' 14/8/07 21:49'!
closeChunkTo: aSocket
	"a zero sized chunk determine and end of chunked data and also response"
	"pack all together to send one TCP packet ony"
	| chunk written |
	chunk := ByteString new: 5.
	chunk at: 1 put: $0 .
	chunk at: 2 put: Character cr. "first crlf ends 0 length line "
	chunk at: 3 put: Character lf.
	chunk at: 4 put: Character cr. "second crlf ends whole response"
	chunk at: 5 put: Character lf.
	written := aSocket writeFrom: chunk asByteArray startingAt: 1 for: chunk size.
	written = chunk size ifFalse: [self error: 'socket write error'].! !

!SwazooBuffer methodsFor: 'accessing' stamp: 'janko 7/26/2008 18:21'!
contents
	"Answer a copy of the receiver's collection, skipping preamble."
	readLimit := readLimit max: position.
	^collection copyFrom: self class preambleSize+1 to: readLimit! !

!SwazooBuffer methodsFor: 'mime boundary' stamp: 'janko 7/26/2008 18:16'!
copyBufferTo: anIndex
	"from current position to desired index"
	| start |
	start := position+1.
	position := anIndex.
	^collection copyFrom: start to: anIndex.! !

!SwazooBuffer methodsFor: 'writing-chunked' stamp: 'mivsek 8/22/2007 20:22'!
flushChunkTo: aSocket
	"a buffer will be sent as a chunk, with hex size in first line then crlf, buffer, then crlf"
	"send a complete chunk in one piece, don't partition TCP sending in too many packets!!"
	| length |
	 self size > 16rFFFF ifTrue: [self error: 'chunk too long!!']. "preamble has no room for bigger"
	length := self size printStringRadix: 16.
	SpEnvironment isSqueak ifTrue: [length := length copyFrom: 4 to: length size]. "trim 16r"
	1 to: length size do: [:inx | collection at: inx put: (length at: inx) asInteger].
	 (length size+1) to: 4 do: [:inx | collection at: inx put: $ asInteger]. "add spaces"
	collection at: 5 put: Character cr asInteger.
	collection at: 6 put: Character lf asInteger.
	"add finishing crlf to buffer"
	self nextPut: Character cr asInteger.
	self nextPut: Character lf asInteger.
	self flushTo: aSocket chunked: true. "flush all at once"! !

!SwazooBuffer methodsFor: 'writing' stamp: ' 14/8/07 21:49'!
flushTo: aSocket
	"actually write to the tcp socket as direclty as possible"
	self flushTo: aSocket chunked: false.
! !

!SwazooBuffer methodsFor: 'private-sending' stamp: 'janko 8/3/2008 17:00'!
flushTo: aSocket chunked: chunkedBoolean
	"actually write to the tcp socket as direclty as possible 
	(directly from stream's instvar collection)"
	| remaining start written |
	remaining := chunkedBoolean 
		ifTrue: [position]  "ensure to send preamble too"
		ifFalse: [position - self class preambleSize]. "skip preamble"
	[remaining > 0] whileTrue: 
		[start := position - remaining + 1.
		written := aSocket 
			writeFrom: collection startingAt: start for: (position - start + 1).
		remaining := remaining - written].
	self writeClear.
	self shouldResize ifTrue: [self resizeBuffer; resizeNil]. "enlarge or shrink buffer if requested"! !

!SwazooBuffer methodsFor: 'mime boundary' stamp: 'janko 7/26/2008 18:17'!
indexOfBoundary: aBoundaryBytes
	"index of boundary start, beeing full boundary or part at the end of buffer. 0 if not found"
	| inx innerInx firstInx |
	inx := position+1.
	[inx <= readLimit] whileTrue:
		[innerInx := 1. firstInx := inx.
		[(aBoundaryBytes at: innerInx) = (collection at: inx)] whileTrue:
			[innerInx = aBoundaryBytes size ifTrue: [^firstInx]. "full boundary found"
			inx = readLimit ifTrue: [^firstInx].  "partial boundary at the edge of buffer found"
			inx := inx + 1. innerInx := innerInx + 1].
		inx := inx + 1].
	^0! !

!SwazooBuffer methodsFor: 'initialize-release' stamp: 'jm 2/8/2009 23:33'!
initDefaultBuffer
	"Squeak specific - writeLimit"
	| size |
	size := self isWrite
		ifTrue: [self class defaultBufferSize + self class preambleSize + 2] "possible chunk crlf"
		ifFalse: [self class defaultBufferSize].
	collection := ByteArray new: size.
	writeLimit := size. "Squeak specific"
	self reset.
	self isWrite ifTrue: [self initPreamble].! !

!SwazooBuffer methodsFor: 'initialize-release' stamp: 'jm 2/8/2009 23:33'!
initLargeBuffer
	"Squeak specific - writeLimit"
	| size |
	size := self isWrite
		ifTrue: [self class largeBufferSize + self class preambleSize + 2] "possible chunk crlf"
		ifFalse: [self class largeBufferSize].
	collection := ByteArray new: size.
	writeLimit := size. "Squeak specific"
	self reset.
	self isWrite ifTrue: [self initPreamble].! !

!SwazooBuffer methodsFor: 'initialize-release' stamp: ' 14/8/07 21:49'!
initPreamble
	"make room for possible chunk 'preamble' = length line"
	position := self class preambleSize.
! !

!SwazooBuffer methodsFor: 'testing' stamp: 'janko 7/26/2008 18:22'!
isBinary
	^collection class == ByteArray! !

!SwazooBuffer methodsFor: 'private-resizing' stamp: 'janko 8/3/2008 16:58'!
isEnlarged
	^collection size > self class defaultBufferSize! !

!SwazooBuffer methodsFor: 'testing' stamp: 'janko 7/26/2008 18:21'!
isFull
	"buffer full or even more"
	^position >= self class defaultBufferSize! !

!SwazooBuffer methodsFor: 'testing' stamp: 'janko 8/3/2008 16:57'!
isRead
	^type = #read! !

!SwazooBuffer methodsFor: 'testing' stamp: 'janko 8/3/2008 16:57'!
isWrite
	^type = #write! !

!SwazooBuffer methodsFor: 'initialize-release' stamp: 'janko 7/26/2008 18:14'!
readClear
	"reset as you'd make another one, just don't replace content array"
	position := 0.
	readLimit := self class defaultBufferSize.! !

!SwazooBuffer methodsFor: 'accessing' stamp: 'janko 7/26/2008 18:16'!
readContents
	^(position = 0 and: [readLimit = self class defaultBufferSize]) "whole buffer?"
		ifTrue: [collection] "avoid copying for performance"
		ifFalse: [collection copyFrom: position+1 to: readLimit]! !

!SwazooBuffer methodsFor: 'reading' stamp: 'janko 8/3/2008 17:01'!
refillFrom: aSocket
	"reset and read from to the tcp socket as direclty as possible 
	(directly from stream's instvar collection)"
	aSocket isNil ifTrue: [^self]. "if SwazooStream is used for tests only"
	self readClear. "just reset pointers, not a collection!! "
	self shouldResize ifTrue: [self resizeBuffer; resizeNil]. "enlarge or shrink buffer if requested"
	readLimit "nr. of actuall bytes read ":= aSocket
		readInto: collection startingAt: 1 for: collection size.
	self atEnd ifTrue: 
		[SwazooStreamNoDataError raiseSignal: 'No data available.  Socket probably closed']! !

!SwazooBuffer methodsFor: 'private-resizing' stamp: 'janko 8/3/2008 16:59'!
resizeBuffer
	"actually do a buffer resize"
	self shouldEnlarge ifTrue: [^self initLargeBuffer].
	self shouldShrink ifTrue: [^self initDefaultBuffer].! !

!SwazooBuffer methodsFor: 'private-resizing' stamp: 'janko 8/3/2008 16:59'!
resizeEnlarge
	"request resizing buffer to larger size at the next fill or flush"
	resize := #enlarge! !

!SwazooBuffer methodsFor: 'private-resizing' stamp: 'janko 8/3/2008 16:59'!
resizeNil
	"nil resizing command"
	resize := nil! !

!SwazooBuffer methodsFor: 'private-resizing' stamp: 'janko 8/3/2008 16:59'!
resizeShrink
	"request shrinking buffer to default size at the next fill or flush"
	resize := #shrink! !

!SwazooBuffer methodsFor: 'initialize-release' stamp: 'janko 8/3/2008 16:56'!
setRead
	type := #read! !

!SwazooBuffer methodsFor: 'initialize-release' stamp: 'janko 8/3/2008 16:56'!
setWrite
	type := #write! !

!SwazooBuffer methodsFor: 'private-resizing' stamp: 'janko 8/3/2008 16:59'!
shouldEnlarge
	"should be resized buffer to larger size?"
	^resize = #enlarge! !

!SwazooBuffer methodsFor: 'private-resizing' stamp: 'janko 8/3/2008 16:58'!
shouldResize
	^resize notNil! !

!SwazooBuffer methodsFor: 'private-resizing' stamp: 'janko 8/3/2008 16:58'!
shouldShrink
	"should be resized buffer to default size?"
	^resize = #shrink! !

!SwazooBuffer methodsFor: 'mime boundary' stamp: 'janko 7/26/2008 18:17'!
signsOfBoundary: aBoundaryBytes
	"detect as fast as possible if any if not all MIME part boundary is present in buffer contents"
	"return number of bundary bytes detected, 0 = no boundary"
	| first index |
	first := aBoundaryBytes first.
	"fast test"
	((self position+1 to: readLimit) contains: [:inx | (collection at: inx) = first]) ifFalse: [^0].
	"full or partial boundary on the edge of buffer test"
	index := self indexOfBoundary: aBoundaryBytes. "index of full, or partial boundary at the edge"
	index = 0 ifTrue: [^0]. "no boundary found"
	readLimit-index >= aBoundaryBytes size ifTrue: [^aBoundaryBytes size]. "full boundary detected"
	^readLimit-index+1  "partial boundary at the end of buffer"! !

!SwazooBuffer methodsFor: 'accessing' stamp: 'janko 7/26/2008 18:21'!
size
	^position - self class preambleSize! !

!SwazooBuffer methodsFor: 'mime boundary' stamp: 'janko 7/26/2008 18:17'!
startsWith: aPartialBoundaryBytes
	"is remaining part of MIME part boundary at the start of buffer?"
	"VW specific!! "
	1 to: aPartialBoundaryBytes size do: [:inx |
		(collection at: position + inx) = (aPartialBoundaryBytes at: inx) ifFalse: [^false] ].
	^true! !

!SwazooBuffer methodsFor: 'initialize-release' stamp: 'janko 7/26/2008 18:12'!
writeClear
	"reset as you'd make another one, just don't replace content array"
	self reset.
	readLimit := 0.
	self initPreamble.! !

!HTTPRequestTest methodsFor: 'requests-gets' stamp: ' 10/7/07 18:48'!
basicGet
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET / HTTP/1.1';
		nextPutLine: 'Host: foo.com';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPRequestTest methodsFor: 'requests-gets' stamp: ' 10/7/07 18:48'!
basicGetHTTP10
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET / HTTP/1.0';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPRequestTest methodsFor: 'requests-gets' stamp: ' 10/7/07 18:48'!
basicGetHTTP10Keepalive
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET / HTTP/1.0';
		nextPutLine: 'Connection: Keep-Alive';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPRequestTest methodsFor: 'requests-gets' stamp: ' 10/7/07 18:48'!
basicHead
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'HEAD / HTTP/1.1';
		nextPutLine: 'Host: foo.com';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPRequestTest methodsFor: 'requests-gets' stamp: ' 10/7/07 18:48'!
fullGet
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET /aaa/bbb/ccc.html?foo=bar&baz=quux HTTP/1.1';
		nextPutLine: 'Connection: Keep-Alive';
		nextPutLine: 'User-Agent: Mozilla/4.72 [en] (X11; I; Linux 2.3.51 i686)';
		nextPutLine: 'Host: foo.com:8888';
		nextPutLine: 'Referer: http://www.bar.com/takeMeThere.html';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPRequestTest methodsFor: 'requests-gets' stamp: 'mivsek 9/28/2007 22:56'!
getMultiValueHeader
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET /aaa/bbb/ccc.html?foo=bar&baz=quux HTTP/1.1';
		nextPutLine: 'Content-Type: multipart/form-data; boundary= --boundary';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPRequestTest methodsFor: 'requests-gets' stamp: ' 10/7/07 18:48'!
portedGet
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET / HTTP/1.1';
		nextPutLine: 'Host: foo.com:8888';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPRequestTest methodsFor: 'testing-other' stamp: 'mivsek 7/21/2007 22:52'!
testMissingContentType
	| requestStream result |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST /foobar HTTP/1.0';
		nextPutLine: 'Host: foo.com';
"	      nextPutLine: 'Content-Type: text/plain'. <-- this is missing!! - and should be for this test"
		nextPutLine: 'Content-Length: 12';
		crlf;
		nextPutLine: 'Hello, World'.
	result := SpExceptionContext 
		for: [(HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)) 
				ensureFullRead] "because of defered post data parsing"
		on: SpError
		do: [:ex | ex].
	self assert: result class == SwazooHTTPPostError.
	^self! !

!HTTPRequestTest methodsFor: 'testing-other' stamp: ' 10/7/07 18:48'!
testNoEqualsQueries
	"The last assert here used to check that 'request queryAt: 'WSDL'' is nil, but a test for an empty string is more consistent with query argument formats."

	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET /test/typed.asmx?WSDL HTTP/1.1';
		nextPutLine: 'Host: foo.com:8888';
		crlf.
	request := HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents).
	self assert: (request includesQuery: 'WSDL').
	self assert: (request queryAt: 'WSDL') isEmpty! !

!SwazooBenchmarks methodsFor: 'initialize-release' stamp: ' 7/5/08 11:08'!
initContent
	| response ws |
	response := HTTPResponse ok.
	response entity: (ByteArray new: self contentSize withAll: 16r55).
	ws := SwazooStream on: String new.
	response writeTo: ws.
	content := ws writeBuffer contents.! !

!TestPseudoSocket methodsFor: 'stream-toServer' stamp: ' 14/4/07 23:10'!
upTo: aCharacter 
	"a ByteString
For some reason, we have to look for a character in a ByteStream - this is a Swazoo thing."

	^(self byteStreamToServer upTo: aCharacter asInteger) asByteString! !
